﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the AISingle project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AISingle.Utility;
using System.Text;

namespace AISingle.ALLDAQ
{
    public class AnalogRange
    {
        private readonly double _physicalResolution;
        private readonly double _offset;

        public string RangeText { get; private set; }
        public string UnitSymbol { get; private set; }

        public AnalogRange(uint physicalUnit, uint digitalMin, uint digitalMax, double physicalMin, double physicalMax)
        {
            _physicalResolution = (physicalMax - physicalMin) / (digitalMax - digitalMin);
            _offset = physicalMin - digitalMin * _physicalResolution;

            UnitSymbol = AdqCommon.GetPhysicalUnitSymbol(physicalUnit);
            InitRangeText(physicalMin, physicalMax, UnitSymbol);
        }

        public double DigitalToPhysical(uint digitalValue)
        {
            return digitalValue * _physicalResolution + _offset;
        }

        private void InitRangeText(double physicalMin, double physicalMax, string unitSymbol)
        {
            StringBuilder rangeTxt = new StringBuilder();

            rangeTxt.Append(AdqCommon.FormatPhysicalValue(physicalMin));
            rangeTxt.Append(unitSymbol);
            rangeTxt.Append("...");
            rangeTxt.Append(AdqCommon.FormatPhysicalValue(physicalMax));
            rangeTxt.Append(unitSymbol);
            RangeText = rangeTxt.ToString();
        }
    }
}
