﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;
using System.Text;
using WriteBufferAO.Utility;

namespace WriteBufferAO.Models
{
    public class AoRange
    {
        private readonly double _physicalResolution;
        private readonly double _digitalOffset;

        public AoRange(uint rangeIndex, uint digitalMin, uint digitalMax, uint physicalUnit, double physicalMin, double physicalMax)
        {
            RangeIndex = rangeIndex;
            DigitalMin = digitalMin;
            DigitalMax = digitalMax;
            PhysicalUnit = physicalUnit;
            PhysicalMin = physicalMin;
            PhysicalMax = physicalMax;
            UnitSymbol = AdqCommon.GetPhysicalUnitSymbol(physicalUnit);
            SetRangeText(UnitSymbol);
            _physicalResolution = (PhysicalMax - PhysicalMin) / (DigitalMax - DigitalMin);
            _digitalOffset = DigitalMin - PhysicalMin / _physicalResolution;
        }

        public uint RangeIndex { get; }
        public uint DigitalMin { get; }
        public uint DigitalMax { get; }
        public uint PhysicalUnit { get; }
        public double PhysicalMin { get; }
        public double PhysicalMax { get; }
        public string UnitSymbol { get; }
        public string RangeText { get; private set; }

        private void SetRangeText(string unitSymbol)
        {
            StringBuilder rangeTxt = new StringBuilder();

            rangeTxt.Append(AdqCommon.FormatPhysicalValue(PhysicalMin));
            rangeTxt.Append(unitSymbol);
            rangeTxt.Append("...");
            rangeTxt.Append(AdqCommon.FormatPhysicalValue(PhysicalMax));
            rangeTxt.Append(unitSymbol);

            RangeText = rangeTxt.ToString();
        }

        public uint PhysicalToDigital(double physicalValue)
        {
            return (uint)Math.Round(physicalValue / _physicalResolution + _digitalOffset);
        }
    }
}
