﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;

namespace WriteBufferAO.Models
{
    public class MexicanHatWaveGenerator : ISignalGenerator
    {
        public string GeneratorIcon
        {
            get
            {
                return "/WriteBufferAO;component/Images/signalMexicanHat.png";
            }
        }

        public string GeneratorName
        {
            get
            {
                return "Mexican hat wavelet";
            }
        }

        public double GetValue(double cycleTime, double signalMinimum, double signalMaximum)
        {
            // https://en.wikipedia.org/wiki/Mexican_hat_wavelet
            double x = cycleTime * 2 - 1.0;
            double xSquared = x * x * 20;
            return (signalMaximum + signalMinimum) / 2.0 + (signalMaximum - signalMinimum) / 2.0 * (1.0 - xSquared) * Math.Pow(Math.E, -xSquared / 2.0);
        }
    }
}
