﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;

namespace WriteBufferAO.Models
{
    public class RandomRectangleWaveGenerator : ISignalGenerator
    {
        private readonly Random _random = new Random();
        private bool _firstSignalPartGenerated;
        private bool _secondSignalPartGenerated;
        private double _randomValue;

        public string GeneratorIcon
        {
            get
            {
                return "/WriteBufferAO;component/Images/signalRandomRectangle.png";
            }
        }

        public string GeneratorName
        {
            get
            {
                return "Random rectangle";
            }
        }
        public double GetValue(double cycleTime, double signalMinimum, double signalMaximum)
        {
            if (cycleTime <= 0.5)
            {
                if (!_firstSignalPartGenerated)
                {
                    _firstSignalPartGenerated = true;
                    _secondSignalPartGenerated = false;
                    _randomValue = _random.NextDouble() * (signalMaximum - signalMinimum) + signalMinimum;
                }
            }
            else
            {
                if (!_secondSignalPartGenerated)
                {
                    _firstSignalPartGenerated = false;
                    _secondSignalPartGenerated = true;
                    _randomValue = _random.NextDouble() * (signalMaximum - signalMinimum) + signalMinimum;
                }
            }
            return _randomValue;
        }
    }
}
