﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferDO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;

namespace WriteBufferDO.Models
{
    class BitShiftRightValueGenerator : IValueGenerator
    {
        public string GeneratorName => "Bit shift right";

        public uint[] GenerateValues(uint portWidthBits, uint numberOfValues, bool lastValueZero)
        {
            uint[] values = new uint[numberOfValues];
            uint idx = 0;

            uint loops = numberOfValues / portWidthBits;
            while (loops > 0)
            {
                SetValues(values, idx, portWidthBits);
                idx += portWidthBits;
                loops--;
            }

            uint remaining = numberOfValues % portWidthBits;
            SetValues(values, idx, remaining);

            if (lastValueZero)
            {
                values[numberOfValues - 1] = 0;
            }
            return values;
        }

        private void SetValues(uint[] values, uint valueIndex, uint numValues)
        {
            if (numValues == 0) return;

            uint value = (uint)(1 << (int)(numValues - 1));
            for (uint bit = 0; bit < numValues; bit++)
            {
                values[valueIndex] = value;
                value >>= 1;
                valueIndex++;
            }
        }
    }
}
