﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferDO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

namespace WriteBufferDO.Models
{
    public class CountUpValueGenerator : IValueGenerator
    {
        public string GeneratorName => "Count up";

        public uint[] GenerateValues(uint portWidthBits, uint numberOfValues, bool lastValueZero)
        {
            uint maxCountUpValues = (1U << (int)portWidthBits);
            uint[] values = new uint[numberOfValues];
            uint idx = 0;

            uint loops = numberOfValues / maxCountUpValues;
            while (loops > 0)
            {
                SetValues(values, idx, maxCountUpValues);
                idx += maxCountUpValues;
                loops--;
            }

            uint remaining = numberOfValues % maxCountUpValues;
            SetValues(values, idx, remaining);

            if (lastValueZero)
            {
                values[numberOfValues -1] = 0;
            }
            return values;
        }

        private void SetValues(uint[] values, uint valueIndex, uint numValues)
        {
            for (uint value = 0; value < numValues; value++)
            {
                values[valueIndex] = value;
                valueIndex++;
            }
        }
    }
}
