﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the ADQInfo project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------
Imports ADQInfo.ALLDAQ.Boards

Public Class AdqCommon
    Private Shared ReadOnly _formatStr As String = "{0:0.######}"

    Public Shared ReadOnly NotOpen As UInteger = AdqErrors.ADQ_ERROR_TYPE_FATAL Or AdqErrors.ADQ_ERROR_NOT_OPEN

    Public Shared Function GetModuleTypeName(moduleType As UInteger) As String
        Return GetDictionaryValue(_moduleTypeNames, moduleType)
    End Function

    Public Shared Function GetPhysicalUnitName(physicalUnit As UInteger) As String
        Return GetDictionaryValue(_physicalUnitNames, physicalUnit)
    End Function

    Public Shared Function GetPhysicalUnitSymbol(physicalUnit As UInteger) As String
        Return GetDictionaryValue(_physicalUnitSymbols, physicalUnit)
    End Function

    Public Shared Function GetClockTypeName(clockType As UInteger) As String
        Return GetDictionaryValue(_clockTypeNames, clockType)
    End Function

    Public Shared Function GetCounterTypeName(counterType As UInteger) As String
        Return GetDictionaryValue(_counterTypeNames, counterType)
    End Function

    Public Shared Function GetCounterClockSourceName(counterClockSource As UInteger) As String
        Return GetDictionaryValue(_counterClockSourceNames, counterClockSource)
    End Function

    Public Shared Function GetBusTypeName(busType As UInteger) As String
        Return GetDictionaryValue(_busTypeNames, busType)
    End Function

    Public Shared Function GetTTLVoltageLevelText(voltageLevel As UInteger) As String
        Return GetDictionaryValue(_ttlVoltageLevels, voltageLevel)
    End Function

    Public Shared Function GetDioDirectionText(dioDirection As UInteger) As String
        Return GetDictionaryValue(_dioDirections, dioDirection)
    End Function

    Public Shared Function GetCalibrationText(calibration As UInteger) As String
        Return GetDictionaryValue(_calibrations, calibration)
    End Function

    Private Shared Function GetDictionaryValue(dictionary As IDictionary(Of UInteger, String), key As UInteger) As String
        Dim stringValue As String = String.Empty
        Dim found As Boolean = dictionary.TryGetValue(key, stringValue)
        If (Not found) Then
            stringValue = String.Format("&H{0:X8}", key)
        End If
        Return stringValue
    End Function

    Private Shared ReadOnly _moduleTypeNames As Dictionary(Of UInteger, String) =
        New Dictionary(Of UInteger, String) From
        {
            {AdqConstants.ADQ_MODULE_TYPE_DIO, "DIO"},
            {AdqConstants.ADQ_MODULE_TYPE_DO, "DO"},
            {AdqConstants.ADQ_MODULE_TYPE_DI, "DI"},
            {AdqConstants.ADQ_MODULE_TYPE_CNTR, "Counter"},
            {AdqConstants.ADQ_MODULE_TYPE_EXTINT, "External Interrupt"},
            {AdqConstants.ADQ_MODULE_TYPE_AI, "AI"},
            {AdqConstants.ADQ_MODULE_TYPE_AO, "AO"},
            {AdqConstants.ADQ_MODULE_TYPE_I2C, "I2C"},
            {AdqConstants.ADQ_MODULE_TYPE_PWM, "PWM"},
            {AdqConstants.ADQ_MODULE_TYPE_FIN, "FIN"},
            {AdqConstants.ADQ_MODULE_TYPE_I2SO, "I2SO"}
        }

    Private Shared ReadOnly _physicalUnitNames As Dictionary(Of UInteger, String) =
            New Dictionary(Of UInteger, String) From
            {
                {AdqConstants.ADQ_PHYSICAL_UNIT_VOLT, "Volt"},
                {AdqConstants.ADQ_PHYSICAL_UNIT_AMPERE, "Ampere"},
                {AdqConstants.ADQ_PHYSICAL_UNIT_OHM, "Ohm"}
            }

    Private Shared ReadOnly _physicalUnitSymbols As Dictionary(Of UInteger, String) =
            New Dictionary(Of UInteger, String) From
            {
                {AdqConstants.ADQ_PHYSICAL_UNIT_VOLT, "V"},
                {AdqConstants.ADQ_PHYSICAL_UNIT_AMPERE, "A"},
                {AdqConstants.ADQ_PHYSICAL_UNIT_OHM, "Ω"}
            }

    Private Shared ReadOnly _clockTypeNames As Dictionary(Of UInteger, String) =
        New Dictionary(Of UInteger, String) From
        {
            {AdqConstants.ADQ_CLOCK_TYPE_HARDWARE, "Hardware"},
            {AdqConstants.ADQ_CLOCK_TYPE_SOFTWARE, "Software"},
            {AdqConstants.ADQ_CLOCK_TYPE_NONE, "None"}
        }

    Private Shared ReadOnly _counterTypeNames As Dictionary(Of UInteger, String) =
        New Dictionary(Of UInteger, String) From
        {
            {AdqConstants.ADQ_CNTR_TYPE_8254, "8254"},
            {AdqConstants.ADQ_CNTR_TYPE_COUNT_DOWN_32_BIT, "Count down 32 Bit"}
        }

    Private Shared ReadOnly _counterClockSourceNames As Dictionary(Of UInteger, String) =
        New Dictionary(Of UInteger, String) From
        {
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_DEFAULT, "Default"},
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_EXTERNAL, "External"},
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ, "Internal 10 MHz"},
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_66_MHZ, "Internal 66 MHz"}
        }

    Private Shared ReadOnly _busTypeNames As Dictionary(Of UInteger, String) =
        New Dictionary(Of UInteger, String) From
        {
            {AdqConstants.ADQ_BUS_TYPE_PCI, "PCI"},
            {AdqConstants.ADQ_BUS_TYPE_USB, "USB"}
        }

    Private Shared ReadOnly _ttlVoltageLevels As Dictionary(Of UInteger, String) =
        New Dictionary(Of UInteger, String) From
        {
            {AdqConstants.ADQ_TTL_VOLTAGE_LEVEL_5_0_VOLTS, String.Format("{0:0.0} V", 5.0)},
            {AdqConstants.ADQ_TTL_VOLTAGE_LEVEL_3_3_VOLTS, String.Format("{0:0.0} V", 3.3)}
        }

    Private Shared ReadOnly _dioDirections As Dictionary(Of UInteger, String) =
        New Dictionary(Of UInteger, String) From
        {
            {AdqConstants.ADQ_DIO_INPUT, "Input"},
            {AdqConstants.ADQ_DIO_OUTPUT, "Output"}
        }

    Private Shared ReadOnly _calibrations As Dictionary(Of UInteger, String) =
        New Dictionary(Of UInteger, String) From
        {
            {AdqConstants.ADQ_CALIBRATION_FACTORY, "Factory"},
            {AdqConstants.ADQ_CALIBRATION_USER_1, "User"}
        }

    Public Shared Function FormatDoubleValue(value As Double, unitText As String) As String
        Dim valueStr As String
        Dim unitStr As String = ""
        Dim tmpVal As Double = Math.Abs(value)
        If (tmpVal = 0) Then
            valueStr = "0"
        ElseIf (tmpVal < 0.000000001) Then
            valueStr = String.Format(_formatStr, value * 1000000000000)
            unitStr = "p"
        ElseIf (tmpVal < 0.000001) Then
            valueStr = String.Format(_formatStr, value * 1000000000)
            unitStr = "n"
        ElseIf (tmpVal < 0.001) Then
            valueStr = String.Format(_formatStr, value * 1000000)
            unitStr = "µ"
        ElseIf (tmpVal < 1) Then
            valueStr = String.Format(_formatStr, value * 1000)
            unitStr = "m"
        ElseIf (tmpVal >= 1000000000) Then
            valueStr = String.Format(_formatStr, value / 1000000000)
            unitStr = "G"
        ElseIf (tmpVal >= 1000000) Then
            valueStr = String.Format(_formatStr, value / 1000000)
            unitStr = "M"
        ElseIf (tmpVal >= 1000) Then
            valueStr = String.Format(_formatStr, value / 1000)
            unitStr = "k"
        Else
            valueStr = String.Format(_formatStr, value)
        End If
        Return valueStr & " " & unitStr & unitText
    End Function

    Public Shared Function FormatBitsPerSecond(value As UInteger) As String
        Dim valueStr As String
        Dim unitStr As String = String.Empty
        If (value >= 1000000000) Then
            valueStr = String.Format("{0}", value / 1000000000)
            unitStr = "G"
        ElseIf (value >= 1000000) Then
            valueStr = String.Format("{0}", value / 1000000)
            unitStr = "M"
        ElseIf (value >= 1000) Then
            valueStr = String.Format("{0}", value / 1000)
            unitStr = "k"
        Else
            valueStr = value.ToString()
        End If
        Return valueStr & " " & unitStr & "bit/s"
    End Function

    Public Shared Function FormatVersion(version As UInteger) As String
        Return String.Format("{0}.{1}.{2}.{3}",
                    (version >> 24) And &HFF,
                    (version >> 16) And &HFF,
                    (version >> 8) And &HFF,
                    (version) And &HFF)
    End Function

    Public Shared Function GetModelName(boardName As String) As String
        Dim posADQ As Integer = boardName.IndexOf("ADQ")
        If (posADQ < 0) Then
            posADQ = 0
        End If

        Dim posEnd As Integer = boardName.IndexOf(" ", posADQ)
        If (posEnd < 0) Then
            posEnd = boardName.Length
        End If

        Return boardName.Substring(posADQ, posEnd - posADQ)
    End Function
End Class
