﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the AISingle project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------
Imports System.Text

Public Class AnalogRange
    Private ReadOnly _physicalResolution As Double
    Private ReadOnly _physicalOffset As Double

    Public Sub New(PhysicalUnit As UInteger, DigitalMin As UInteger, DigitalMax As UInteger, PhysicalMin As Double, PhysicalMax As Double)
        _physicalResolution = (PhysicalMax - PhysicalMin) / (DigitalMax - DigitalMin)
        _physicalOffset = PhysicalMin - DigitalMin * _physicalResolution
        UnitSymbol = AdqCommon.GetPhysicalUnitSymbol(PhysicalUnit)
        RangeText = GetRangeText(PhysicalMin, PhysicalMax)
    End Sub

    Public ReadOnly Property RangeText As String
    Public ReadOnly Property UnitSymbol As String

    Public Function DigitalToPhysical(digitalValue As UInteger) As Double
        Return digitalValue * _physicalResolution + _physicalOffset
    End Function

    Private Function GetRangeText(physicalMin As Double, physicalMax As Double) As String
        Dim rangeTxt = New StringBuilder()

        rangeTxt.Append(FormatPhysicalValue(physicalMin))
        rangeTxt.Append(UnitSymbol)
        rangeTxt.Append("...")
        rangeTxt.Append(FormatPhysicalValue(physicalMax))
        rangeTxt.Append(UnitSymbol)
        Return rangeTxt.ToString()
    End Function

    Private Function FormatPhysicalValue(physicalValue As Double) As String
        Dim physicalTxt As String
        Dim fmt As String = "{0:0.##}{1}"
        Dim tmpVal As Double = Math.Abs(physicalValue)
        If tmpVal = 0 Then
            physicalTxt = "0"
        ElseIf tmpVal < 0.001 Then
            physicalTxt = String.Format(fmt, physicalValue * 1000000, "µ")
        ElseIf tmpVal < 1 Then
            physicalTxt = String.Format(fmt, physicalValue * 1000, "m")
        Else
            physicalTxt = String.Format(fmt, Math.Round(physicalValue, 6), "")
        End If
        Return physicalTxt
    End Function
End Class
