﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the AOSingle project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Text

Public Class AoRange
    Private ReadOnly _physicalResolution As Double
    Private ReadOnly _digitalOffset As Double

    Public Sub New(rangeIndex As UInteger, digitalMin As UInteger, digitalMax As UInteger, physicalUnit As UInteger, physicalMin As Double, physicalMax As Double)
        Me.RangeIndex = rangeIndex
        Me.DigitalMin = digitalMin
        Me.DigitalMax = digitalMax
        Me.PhysicalUnit = physicalUnit
        Me.PhysicalMin = physicalMin
        Me.PhysicalMax = physicalMax
        UnitSymbol = AdqCommon.GetPhysicalUnitSymbol(physicalUnit)
        RangeText = GetRangeText(UnitSymbol)
        _physicalResolution = (physicalMax - physicalMin) / (digitalMax - digitalMin)
        _digitalOffset = digitalMin - physicalMin / _physicalResolution
        _currentDigitalValue = PhysicalToDigital(0)
    End Sub

    Public ReadOnly Property RangeIndex As UInteger
    Public ReadOnly Property DigitalMin As UInteger
    Public ReadOnly Property DigitalMax As UInteger
    Public ReadOnly Property PhysicalUnit As UInteger
    Public ReadOnly Property PhysicalMin As Double
    Public ReadOnly Property PhysicalMax As Double
    Public ReadOnly Property UnitSymbol As String
    Public ReadOnly Property RangeText As String

    Private _currentDigitalValue As UInteger
    Public Property CurrentDigitalValue As UInteger
        Get
            Return _currentDigitalValue
        End Get
        Set
            _currentDigitalValue = Value
        End Set
    End Property

    Private Function GetRangeText(unitSymbol As String) As String
        Dim rangeTxt As StringBuilder = New StringBuilder()

        rangeTxt.Append(AdqCommon.FormatPhysicalValue(PhysicalMin))
        rangeTxt.Append(unitSymbol)
        rangeTxt.Append("...")
        rangeTxt.Append(AdqCommon.FormatPhysicalValue(PhysicalMax))
        rangeTxt.Append(unitSymbol)

        Return rangeTxt.ToString()
    End Function

    Public Function PhysicalToDigital(physicalValue As Double) As UInteger
        Return CUInt(Math.Round(physicalValue / _physicalResolution + _digitalOffset))
    End Function

    Public Function DigitalToPhysical(digitalValue As UInteger) As Double
        Return _physicalResolution * digitalValue + PhysicalMin
    End Function

    Public Function ValidatePhysicalValue(physicalValue As Double) As String
        Dim msg As String = String.Empty
        If physicalValue < PhysicalMin OrElse physicalValue > PhysicalMax Then

            msg = "The physical value must be within the range limits."
        End If
        Return msg
    End Function

    Public Function ValidateDigitalValue(digital As UInteger) As String
        Dim msg As String = String.Empty
        If digital < DigitalMin OrElse digital > DigitalMax Then
            msg = "The digital value must be within the range limits."
        End If
        Return msg
    End Function
End Class
