﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file is part of the CommonSettings project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports CommonSettings.ALLDAQ.Boards

Public Class AnalogModule
    Inherits AdqModule

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger, moduleType As UInteger)
        MyBase.New(boardIndex, moduleIndex)

        BuildModuleName(boardName, moduleType)
    End Sub

    Private _moduleName As String
    Public ReadOnly Property ModuleName As String
        Get
            Return _moduleName
        End Get
    End Property

    Private Sub BuildModuleName(boardName As String, moduleType As UInteger)
        _moduleName = String.Format("{0}, Board {1}, Module {2}, Type {3}",
            AdqCommon.GetModelName(boardName),
            BoardIndex,
            ModuleIndex,
            AdqCommon.GetModuleTypeText(moduleType))
    End Sub

    Public Function GetSelectedCalibration(ByRef calibrationId As UInteger) As Boolean
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE
        adqError = AdqDriver.adqAnalogGetSelectedCalibration(BoardIndex, ModuleIndex, calibrationId)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqAIGetSelectedCalibration", adqError)
        End If
        Return adqError = AdqErrors.ADQ_ERROR_NONE
    End Function

    Public Function SetSelectedCalibration(calibrationId As UInteger) As Boolean
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE
        adqError = AdqDriver.adqAnalogSelectCalibration(BoardIndex, ModuleIndex, calibrationId)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqAnalogSetSelectedCalibration", adqError)
        End If
        Return adqError = AdqErrors.ADQ_ERROR_NONE
    End Function
End Class
