﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file is part of the CommonSettings project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports CommonSettings.ALLDAQ.Boards

Public Class DigitalInputModule
    Inherits AdqModule

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger,
            ByRef adqDiModuleInfo As AdqDriver.ADQDIModuleInfo)
        MyBase.New(boardIndex, moduleIndex)

        BuildModuleName(boardName)
        FillFilterTimes(adqDiModuleInfo)
    End Sub

    Private _moduleName As String
    Public ReadOnly Property ModuleName As String
        Get
            Return _moduleName
        End Get
    End Property

    Public ReadOnly Property FilterTimes As List(Of ULong) = New List(Of ULong)()

    Private Sub BuildModuleName(boardName As String)
        _moduleName = String.Format("{0}, Board {1}, Module {2}",
            AdqCommon.GetModelName(boardName),
            BoardIndex,
            ModuleIndex)
    End Sub

    Private Sub FillFilterTimes(adqDiModuleInfo As AdqDriver.ADQDIModuleInfo)
        Dim idx As Integer = 0
        While idx < adqDiModuleInfo.NumberOfFilters
            FilterTimes.Add(adqDiModuleInfo.FilterTimePicoseconds(idx))
            idx += 1
        End While
    End Sub

    Public Function GetDigitalFilter(ByRef filterPicoseconds As ULong) As Boolean
        Dim adqError As UInteger = AdqDriver.adqDIOGetFilterPicoseconds(BoardIndex, ModuleIndex, filterPicoseconds)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqDIOGetFilterPicoseconds", adqError)
        End If
        Return adqError = AdqErrors.ADQ_ERROR_NONE
    End Function

    Public Function SetDigitalFilter(filterPicoseconds As ULong) As Boolean
        Dim adqError As UInteger = AdqDriver.adqDIOSetFilterPicoseconds(BoardIndex, ModuleIndex, filterPicoseconds)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqDIOSetFilterPicoseconds", adqError)
        End If
        Return adqError = AdqErrors.ADQ_ERROR_NONE
    End Function
End Class
