﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'      This file contains a programming example for Intel counter 82C54 ICs
'      on ALLDAQ Data Acquisition Boards
'-----------------------------------------------------------------------------

Imports Counter8254.ALLDAQ.Boards

Public Class CounterModule
    Private ReadOnly _boardIndex As UInteger

    Public Property CounterMaxValue As UInteger

    Sub New(boardIndex As UInteger, moduleIndex As UInteger, widthBits As UInteger, clockSources As UInteger)
        _boardIndex = boardIndex
        Me.ModuleIndex = moduleIndex
        CounterMaxValue = GetCounterMaxValue(widthBits)
        SetClockSourceValues(clockSources)
    End Sub

    Public ReadOnly Property ModuleIndex As UInteger
    Public ReadOnly Property ClockSourceValues As List(Of UInteger) = New List(Of UInteger)

    Private Function GetCounterMaxValue(widthBits As UInteger) As UInteger
        If widthBits < 32 Then
            Return (1UI << CInt(widthBits)) - 1UI
        End If
        Return UInteger.MaxValue
    End Function

    Public Function Config(clockSource As UInteger, counterMode As UInteger, startValue As UInteger) As UInteger
        Return AdqDriver.adqCNTRConfig(_boardIndex, ModuleIndex, clockSource, counterMode, startValue)
    End Function

    Public Function ReadSingle(ByRef value As UInteger) As UInteger
        Return AdqDriver.adqReadSingle(_boardIndex, ModuleIndex, value)
    End Function

    Public Function Reset() As UInteger
        Return AdqDriver.adqResetModule(_boardIndex, ModuleIndex)
    End Function

    Private Sub SetClockSourceValues(clockSources As UInteger)
        AddClockSourceValue(clockSources, AdqConstants.ADQ_CNTR_CLOCK_SOURCE_EXTERNAL)
        AddClockSourceValue(clockSources, AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ)
    End Sub

    Private Sub AddClockSourceValue(clockSources As UInteger, clockSourceConstant As UInteger)
        If (clockSources And clockSourceConstant) = clockSourceConstant Then
            ClockSourceValues.Add(clockSourceConstant)
        End If
    End Sub
End Class
