﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'      This file contains a programming example for Intel counter 82C54 ICs
'      on ALLDAQ Data Acquisition Boards
'-----------------------------------------------------------------------------

Imports Counter8254.ALLDAQ.Boards

Public Class FormMain
    Private _selectedCountersIndex As Integer
    Private _moduleLocator As ModuleLocator
    Private _cntr8254Modules As List(Of Counter8254Modules)
    Private _counter0 As CounterModule
    Private _counter1 As CounterModule
    Private _counter2 As CounterModule
    Private _counterUi0 As CounterUI
    Private _counterUi1 As CounterUI
    Private _counterUi2 As CounterUI

    'Dim counters As Counter8254Modules

    Private Sub FormMain_Shown(sender As Object, e As EventArgs) Handles MyBase.Shown
        InitCounterUiElements()
        EnableUi(False)
        Try
            _cntr8254Modules = New List(Of Counter8254Modules)
            _moduleLocator = New ModuleLocator(_cntr8254Modules, AdqConstants.ADQ_MODULE_TYPE_CNTR, AddressOf OnModuleInfoTextChanged)
            _moduleLocator.FindAdqModules()
            FillCountersCombobox()

            _selectedCountersIndex = -1
            If coBoxCounters.Items.Count > 0 Then
                coBoxCounters.SelectedIndex = 0
            End If
        Catch ex As DllNotFoundException
            AppendInfoText("The device driver is not properly installed.")
            textBoxInfo.AppendText(ex.Message & Environment.NewLine)
        End Try
    End Sub

    Private Sub InitCounterUiElements()
        _counterUi0 = New CounterUI With {
            .GrpBoxModuleNumber = grpBoxCounter0,
            .CoBoxCounterMode = coBoxMode0,
            .CoBoxClockSource = coBoxClkSource0,
            .CoBoxStartValue = coBoxCntrStartValue0,
            .TxtBoxCounterValue = txtBoxCtrValue0,
            .ButtonSetConfig = btnSetConfig0,
            .ButtonRead = btnRead0,
            .ButtonReset = btnReset0,
            .ButtonReadInInterval = btnReadInInterval0,
            .ButtonStop = btnStop0
        }

        _counterUi1 = New CounterUI With {
            .GrpBoxModuleNumber = grpBoxCounter1,
            .CoBoxCounterMode = coBoxMode1,
            .CoBoxClockSource = coBoxClkSource1,
            .CoBoxStartValue = coBoxCntrStartValue1,
            .TxtBoxCounterValue = txtBoxCtrValue1,
            .ButtonSetConfig = btnSetConfig1,
            .ButtonRead = btnRead1,
            .ButtonReset = btnReset1,
            .ButtonReadInInterval = btnReadInInterval1,
            .ButtonStop = btnStop1
        }

        _counterUi2 = New CounterUI With {
            .GrpBoxModuleNumber = grpBoxCounter2,
            .CoBoxCounterMode = coBoxMode2,
            .CoBoxClockSource = coBoxClkSource2,
            .CoBoxStartValue = coBoxCntrStartValue2,
            .TxtBoxCounterValue = txtBoxCtrValue2,
            .ButtonSetConfig = btnSetConfig2,
            .ButtonRead = btnRead2,
            .ButtonReset = btnReset2,
            .ButtonReadInInterval = btnReadInInterval2,
            .ButtonStop = btnStop2
        }
    End Sub

    Private Sub FillCountersCombobox()
        coBoxCounters.Items.Clear()
        For Each counters8254 As Counter8254Modules In _cntr8254Modules
            coBoxCounters.Items.Add(counters8254.ToString())
        Next
    End Sub

    Private Sub OnModuleInfoTextChanged(sender As Object, e As InfoTextChangedEventArgs)
        textBoxInfo.AppendText(e.InfoText & Environment.NewLine)
    End Sub

    Private Sub CoBoxCounters_SelectedIndexChanged(sender As Object, e As EventArgs) Handles coBoxCounters.SelectedIndexChanged
        If _selectedCountersIndex = coBoxCounters.SelectedIndex Then
            Return
        End If

        StopIntervalTimers()
        _selectedCountersIndex = coBoxCounters.SelectedIndex
        Dim cntrModules As Counter8254Modules = Nothing
        If coBoxCounters.SelectedIndex >= 0 Then
            cntrModules = _cntr8254Modules(coBoxCounters.SelectedIndex)
        End If
        ChangeCountersSelection(cntrModules)
    End Sub

    Private Sub ChangeCountersSelection(counterModules As Counter8254Modules)
        If counterModules IsNot Nothing Then
            _counter0 = counterModules.GetCounter(0)
            _counter1 = counterModules.GetCounter(1)
            _counter2 = counterModules.GetCounter(2)
        Else
            _counter0 = Nothing
            _counter1 = Nothing
            _counter2 = Nothing
        End If
        InitCounterUiElements(_counter0, _counterUi0)
        InitCounterUiElements(_counter1, _counterUi1)
        InitCounterUiElements(_counter2, _counterUi2)
    End Sub

    Private Sub InitCounterUiElements(counter As CounterModule, counterUI As CounterUI)
        counterUI.TxtBoxCounterValue.Text = "0"
        counterUI.CoBoxClockSource.Items.Clear()
        If counter IsNot Nothing Then
            counterUI.GrpBoxModuleNumber.Text = String.Format(" Module {0} ", counter.ModuleIndex)
            counterUI.CoBoxStartValue.Text = String.Format("{0}", counter.CounterMaxValue)
            FillCounterClockSources(counter, counterUI)
            counterUI.CoBoxCounterMode.SelectedIndex = 0
            counterUI.EnableUI(True)
        Else
            counterUI.GrpBoxModuleNumber.Text = " Not available "
            counterUI.CoBoxStartValue.Text = String.Empty
            counterUI.CoBoxCounterMode.SelectedIndex = -1
            counterUI.EnableUI(False)
        End If
    End Sub

    Private Sub FillCounterClockSources(counter As CounterModule, counterUI As CounterUI)
        For i As Integer = 0 To counter.ClockSourceValues.Count - 1
            Dim item = New ClockSourceComboBoxItem(counter.ClockSourceValues(i))
            counterUI.CoBoxClockSource.Items.Add(item)
        Next
        counterUI.CoBoxClockSource.SelectedIndex = 0
    End Sub

    Private Sub BtnSetConfig0_Click(sender As Object, e As EventArgs) Handles btnSetConfig0.Click
        StopIntervalTimer(Timer0, _counter0, _counterUi0)
        Try
            ConfigureCounter(_counter0, _counterUi0)
        Catch ex As Exception
            MessageBox.Show(ex.Message, Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub BtnSetConfig1_Click(sender As Object, e As EventArgs) Handles btnSetConfig1.Click
        StopIntervalTimer(Timer1, _counter1, _counterUi1)
        Try
            ConfigureCounter(_counter1, _counterUi1)
        Catch ex As Exception
            MessageBox.Show(ex.Message, Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub BtnSetConfig2_Click(sender As Object, e As EventArgs) Handles btnSetConfig2.Click
        StopIntervalTimer(Timer2, _counter2, _counterUi2)
        Try
            ConfigureCounter(_counter2, _counterUi2)
        Catch ex As Exception
            MessageBox.Show(ex.Message, Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Function ConfigureCounter(counter As CounterModule, counterUI As CounterUI) As Boolean
        Dim clockSource As UInteger = CType(counterUI.CoBoxClockSource.SelectedItem, ClockSourceComboBoxItem).GetValue
        Dim counterMode As UInteger = AdqCommon.Counter8254Modes(counterUI.CoBoxCounterMode.SelectedIndex)
        Dim startValue As UInteger = UInteger.Parse(counterUI.CoBoxStartValue.Text)
        Return WriteCounterConfiguration(counter, clockSource, counterMode, startValue, counterUI.CoBoxCounterMode.Text)
    End Function

    Private Function WriteCounterConfiguration(counter As CounterModule,
                                               clockSource As UInteger,
                                               counterMode As UInteger,
                                               startValue As UInteger,
                                               ctrModeName As String) As Boolean
        Dim adqError As UInteger = counter.Config(clockSource, counterMode, startValue)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            DisplayError(counter.ModuleIndex, adqError)
            Return False
        End If

        DisplayConfiguration(counter.ModuleIndex, ctrModeName, startValue)
        Return True
    End Function

    Private Sub BtnRead0_Click(sender As Object, e As EventArgs) Handles btnRead0.Click
        Dim adqError As UInteger = ReadCounter(_counter0, _counterUi0)
        DisplayReadSingle(_counter0.ModuleIndex, adqError)
    End Sub

    Private Sub BtnRead1_Click(sender As Object, e As EventArgs) Handles btnRead1.Click
        Dim adqError As UInteger = ReadCounter(_counter1, _counterUi1)
        DisplayReadSingle(_counter1.ModuleIndex, adqError)
    End Sub

    Private Sub BtnRead2_Click(sender As Object, e As EventArgs) Handles btnRead2.Click
        Dim adqError As UInteger = ReadCounter(_counter2, _counterUi2)
        DisplayReadSingle(_counter2.ModuleIndex, adqError)
    End Sub

    Private Function ReadCounter(counter As CounterModule, counterUI As CounterUI) As UInteger
        Dim value As UInteger = 0
        Dim adqError As UInteger = counter.ReadSingle(value)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            counterUI.TxtBoxCounterValue.Text = value.ToString()
        Else
            counterUI.TxtBoxCounterValue.Text = "Error " & adqError.ToString()
        End If
        Return adqError
    End Function

    Private Sub BtnReset0_Click(sender As Object, e As EventArgs) Handles btnReset0.Click
        StopIntervalTimer(Timer0, _counter0, _counterUi0)
        ResetCounterModule(_counter0, _counterUi0)
    End Sub

    Private Sub BtnReset1_Click(sender As Object, e As EventArgs) Handles btnReset1.Click
        StopIntervalTimer(Timer1, _counter1, _counterUi1)
        ResetCounterModule(_counter1, _counterUi1)
    End Sub

    Private Sub BtnReset2_Click(sender As Object, e As EventArgs) Handles btnReset2.Click
        StopIntervalTimer(Timer2, _counter2, _counterUi2)
        ResetCounterModule(_counter2, _counterUi2)
    End Sub

    Private Sub ResetCounterModule(counter As CounterModule, counterUI As CounterUI)
        Dim adqError As UInteger = counter.Reset()
        counterUI.TxtBoxCounterValue.Text = "0"
        DisplayReset(counter.ModuleIndex, adqError)
    End Sub

    Private Sub BtnReadInInterval0_Click(sender As Object, e As EventArgs) Handles btnReadInInterval0.Click
        Try
            Dim canStart As Boolean = ConfigureCounter(_counter0, _counterUi0)
            StartReadInInterval(canStart, Timer0, _counter0, _counterUi0)
        Catch ex As Exception
            MessageBox.Show(ex.Message, Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub BtnReadInInterval1_Click(sender As Object, e As EventArgs) Handles btnReadInInterval1.Click
        Try
            Dim canStart As Boolean = ConfigureCounter(_counter1, _counterUi1)
            StartReadInInterval(canStart, Timer1, _counter1, _counterUi1)
        Catch ex As Exception
            MessageBox.Show(ex.Message, Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub BtnReadInInterval2_Click(sender As Object, e As EventArgs) Handles btnReadInInterval2.Click
        Try
            Dim canStart As Boolean = ConfigureCounter(_counter2, _counterUi2)
            StartReadInInterval(canStart, Timer2, _counter2, _counterUi2)
        Catch ex As Exception
            MessageBox.Show(ex.Message, Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub StartReadInInterval(canStart As Boolean, timer As Timer, counter As CounterModule, counterUI As CounterUI)
        counterUI.ButtonReadInInterval.Enabled = Not canStart
        If canStart Then
            DisplayReadInInterval("Start", counter.ModuleIndex)
            timer.Enabled = True
        End If
    End Sub

    Private Sub Timer0_Tick(sender As Object, e As EventArgs) Handles Timer0.Tick
        Dim adqError As UInteger = ReadCounter(_counter0, _counterUi0)
        If IsCriticalError(adqError) Then
            StopIntervalTimer(Timer0, _counter0, _counterUi0)
            DisplayError(_counter0.ModuleIndex, adqError)
        End If
    End Sub

    Private Sub Timer1_Tick(sender As Object, e As EventArgs) Handles Timer1.Tick
        Dim adqError As UInteger = ReadCounter(_counter1, _counterUi1)
        If IsCriticalError(adqError) Then
            StopIntervalTimer(Timer1, _counter1, _counterUi1)
            DisplayError(_counter1.ModuleIndex, adqError)
        End If
    End Sub

    Private Sub Timer2_Tick(sender As Object, e As EventArgs) Handles Timer2.Tick
        Dim adqError As UInteger = ReadCounter(_counter2, _counterUi2)
        If IsCriticalError(adqError) Then
            StopIntervalTimer(Timer2, _counter2, _counterUi2)
            DisplayError(_counter2.ModuleIndex, adqError)
        End If
    End Sub

    Private Function IsCriticalError(adqError As UInteger) As Boolean
        Return adqError <> AdqErrors.ADQ_ERROR_NONE AndAlso adqError <> AdqErrors.ADQ_ERROR_INTERNAL
    End Function

    Private Sub BtnStop0_Click(sender As Object, e As EventArgs) Handles btnStop0.Click
        StopIntervalTimer(Timer0, _counter0, _counterUi0)
    End Sub

    Private Sub BtnStop1_Click(sender As Object, e As EventArgs) Handles btnStop1.Click
        StopIntervalTimer(Timer1, _counter1, _counterUi1)
    End Sub

    Private Sub BtnStop2_Click(sender As Object, e As EventArgs) Handles btnStop2.Click
        StopIntervalTimer(Timer2, _counter2, _counterUi2)
    End Sub

    Private Sub StopIntervalTimer(timer As Timer, counter As CounterModule, counterUI As CounterUI)
        If timer.Enabled Then
            timer.Enabled = False
            If counter IsNot Nothing Then
                DisplayReadInInterval("Stop", counter.ModuleIndex)
            End If
        End If
        counterUI.ButtonReadInInterval.Enabled = counter IsNot Nothing
    End Sub

    Private Sub StopIntervalTimers()
        StopIntervalTimer(Timer0, _counter0, _counterUi0)
        StopIntervalTimer(Timer1, _counter1, _counterUi1)
        StopIntervalTimer(Timer2, _counter2, _counterUi2)
    End Sub

    Private Sub CloseAdqDriver()
        If _moduleLocator IsNot Nothing Then
            StopIntervalTimers()
            _moduleLocator.CloseAdqDriver()
        End If
    End Sub

    Private Sub DisplayConfiguration(moduleIndex As UInteger, ctrMode As String, startValue As UInteger)
        AppendInfoText("Configure module " & moduleIndex.ToString() _
            & ", Counter mode: " & ctrMode _
            & ", Start value: " & startValue.ToString())
    End Sub

    Private Sub DisplayReadSingle(moduleIndex As UInteger, adqError As UInteger)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            AppendInfoText("Read single value from module " & moduleIndex.ToString())
            Return
        End If
        DisplayError(moduleIndex, adqError)
    End Sub

    Private Sub DisplayReadInInterval(startOrStop As String, moduleIndex As UInteger)
        AppendInfoText(startOrStop & " interval reading from module " & moduleIndex.ToString())
    End Sub

    Private Sub DisplayReset(moduleIndex As UInteger, adqError As UInteger)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            AppendInfoText("Reset module " & moduleIndex.ToString())
            Return
        End If
        DisplayErrorText("adqReset", adqError)
    End Sub

    Private Sub DisplayError(moduleIndex As UInteger, adqError As UInteger)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            AppendInfoText(String.Format("Error on counter module {0}, Error &H{1:X8}",
                           moduleIndex, adqError))
        End If
    End Sub
    Private Sub DisplayErrorText(functionName As String, adqError As UInteger)
        AppendInfoText(
            String.Format("Function {0} returned Error number: &H{1:X}",
                functionName,
                adqError))
    End Sub

    Private Sub AppendInfoText(infoText As String)
        textBoxInfo.AppendText("[" + Date.Now.ToString("HH:mm:ss.fff") + "] " & infoText & Environment.NewLine)
    End Sub

    Private Sub EnableUi(enable As Boolean)
        _counterUi0.EnableUI((_counter0 IsNot Nothing) AndAlso enable)
        _counterUi1.EnableUI((_counter1 IsNot Nothing) AndAlso enable)
        _counterUi2.EnableUI((_counter2 IsNot Nothing) AndAlso enable)
    End Sub

    Private Sub FormMain_FormClosing(sender As Object, e As FormClosingEventArgs) Handles MyBase.FormClosing
        CloseAdqDriver()
    End Sub

    Private Sub BtnClose_Click(sender As Object, e As EventArgs) Handles btnClose.Click
        Close()
    End Sub

    Private Sub BtnInfo_Click(sender As Object, e As EventArgs) Handles btnInfo.Click
        AboutBox.ShowDialog()
    End Sub

    Private Sub BtnClear_Click(sender As Object, e As EventArgs) Handles btnClear.Click
        textBoxInfo.Clear()
    End Sub
End Class
