﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferAO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Text

Public Class AoRange
    Private ReadOnly _physicalResolution As Double
    Private ReadOnly _digitalOffset As Double

    Public Sub New(digitalMin As UInteger, digitalMax As UInteger, physicalUnit As UInteger, physicalMin As Double, physicalMax As Double)
        Me.PhysicalMin = physicalMin
        Me.PhysicalMax = physicalMax
        UnitSymbol = AdqCommon.GetPhysicalUnitSymbol(physicalUnit)
        SetRangeText(physicalMin, physicalMax)
        _physicalResolution = (physicalMax - physicalMin) / (digitalMax - digitalMin)
        _digitalOffset = digitalMin - physicalMin / _physicalResolution
    End Sub
    Public ReadOnly Property UnitSymbol As String

    Private _rangeText As String
    Public ReadOnly Property RangeText As String
        Get
            Return _rangeText
        End Get
    End Property

    Public ReadOnly Property PhysicalMin As Double
    Public ReadOnly Property PhysicalMax As Double

    Private Sub SetRangeText(physicalMin As Double, physicalMax As Double)
        Dim rangeTxt As StringBuilder = New StringBuilder()

        rangeTxt.Append(AdqCommon.FormatPhysicalValue(physicalMin))
        rangeTxt.Append(UnitSymbol)
        rangeTxt.Append("...")
        rangeTxt.Append(AdqCommon.FormatPhysicalValue(physicalMax))
        rangeTxt.Append(UnitSymbol)

        _rangeText = rangeTxt.ToString()
    End Sub

    Public Function PhysicalToDigital(physicalValue As Double) As UInteger
        Return CUInt(Math.Round(physicalValue / _physicalResolution + _digitalOffset))
    End Function
End Class
