﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferAO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Public Class GaussianNoiseSignalGenerator
    Implements ISignalGenerator

    Private ReadOnly _random As Random = New Random()

    Public ReadOnly Property GeneratorIcon As String Implements ISignalGenerator.GeneratorIcon
        Get
            Return "/WriteBufferAO;component/Images/signalGaussNoise.png"
        End Get
    End Property

    Public ReadOnly Property GeneratorName As String Implements ISignalGenerator.GeneratorName
        Get
            Return "Gaussian noise"
        End Get
    End Property

    Public Function GetValue(cycleTime As Double, signalMinimum As Double, signalMaximum As Double) As Double Implements ISignalGenerator.GetValue

        'https://en.wikipedia.org/wiki/Marsaglia_polar_method
        Dim stdDev As Double = (signalMaximum - signalMinimum) / 8.0
        Dim mean As Double = (signalMaximum + signalMinimum) / 2.0

        Dim u1 As Double
        Dim u2 As Double
        Dim r As Double
        Dim value As Double
        While True
            Do
                u1 = 2 * _random.NextDouble() - 1
                u2 = 2 * _random.NextDouble() - 1
                r = u1 * u1 + u2 * u2
            Loop While (r >= 1) OrElse (r = 0)

            Dim polar As Double = Math.Sqrt(-2.0 * Math.Log(r) / r)
            value = polar * u1 * stdDev + mean
            If (value >= signalMinimum) AndAlso (value <= signalMaximum) Then
                Return value
            End If
        End While
        Return value
    End Function
End Class
