﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferAO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Public Class PoissonWaveletGenerator
    Implements ISignalGenerator

    Public ReadOnly Property GeneratorIcon As String Implements ISignalGenerator.GeneratorIcon
        Get
            Return "/WriteBufferAO;component/Images/signalPoissonWavelet.png"
        End Get
    End Property

    Public ReadOnly Property GeneratorName As String Implements ISignalGenerator.GeneratorName
        Get
            Return "Poisson wavelet"
        End Get
    End Property

    Public Function GetValue(cycleTime As Double, signalMinimum As Double, signalMaximum As Double) As Double Implements ISignalGenerator.GetValue
        ' https://en.wikipedia.org/wiki/Poisson_wavelet
        Dim x As Double = cycleTime * 18.0 - 9.0
        Dim xSquared As Double = x * x
        Return (signalMaximum + signalMinimum) / 2.0 + (signalMaximum - signalMinimum) / 2.0 * ((1 - xSquared) / Math.Pow(1 + xSquared, 2))
    End Function
End Class
