﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferAO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Public Class SincWaveGenerator
    Implements ISignalGenerator

    Public ReadOnly Property GeneratorIcon As String Implements ISignalGenerator.GeneratorIcon
        Get
            Return "/WriteBufferAO;component/Images/signalSinc.png"
        End Get
    End Property

    Public ReadOnly Property GeneratorName As String Implements ISignalGenerator.GeneratorName
        Get
            Return "Sinc function"
        End Get
    End Property

    Public Function GetValue(cycleTime As Double, signalMinimum As Double, signalMaximum As Double) As Double Implements ISignalGenerator.GetValue
        ' https://en.wikipedia.org/wiki/Sinc_function
        Dim offset As Double = (signalMaximum + signalMinimum) / 2.0
        Dim factor As Double = (signalMaximum - signalMinimum) / 2.0

        Dim x As Double = AdqCommon.TwoPI * (cycleTime - 0.5) * 10
        If x = 0 Then
            Return offset + factor
        End If
        Return offset + factor * Math.Sin(x) / x
    End Function
End Class
