﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferDO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Public Class BitShiftRightValueGenerator
    Implements IValueGenerator
    Public ReadOnly Property GeneratorName As String Implements IValueGenerator.GeneratorName
        Get
            Return "Bit shift right"
        End Get
    End Property

    Public Function GenerateValues(portWidthBits As UInteger, numberOfValues As UInteger, lastValueZero As Boolean) As UInteger() _
        Implements IValueGenerator.GenerateValues

        Dim values() = New UInteger(CInt(numberOfValues - 1UI)) {}
        Dim idx As UInteger = 0
        Dim loops As UInteger = numberOfValues \ portWidthBits

        While loops > 0
            SetValues(values, idx, portWidthBits)
            idx += portWidthBits
            loops -= 1UI
        End While

        Dim remaining As UInteger = numberOfValues Mod portWidthBits
        SetValues(values, idx, remaining)

        If lastValueZero Then
            values(CInt(numberOfValues - 1)) = 0
        End If
        Return values
    End Function

    Private Sub SetValues(ByRef values As UInteger(), ByVal index As UInteger, ByVal numValues As UInteger)
        Dim val As UInteger = CUInt(1 << CInt(numValues - 1))
        Dim bit As Integer = 0
        While bit < numValues
            values(CInt(index)) = val
            index += 1UI
            val >>= 1
            bit += 1
        End While
    End Sub
End Class
