﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferDO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Public Class CountUpValueGenerator
    Implements IValueGenerator

    Public ReadOnly Property GeneratorName As String Implements IValueGenerator.GeneratorName
        Get
            Return "Count up"
        End Get
    End Property

    Public Function GenerateValues(portWidthBits As UInteger, numberOfValues As UInteger, lastValueZero As Boolean) As UInteger() _
        Implements IValueGenerator.GenerateValues

        Dim maxCountUpValues As UInteger = GetMaxNumberOfValues(portWidthBits)
        Dim values As UInteger() = New UInteger(CInt(numberOfValues - 1UI)) {}
        Dim idx As UInteger = 0

        Dim loops As UInteger = numberOfValues \ maxCountUpValues
        While loops > 0
            SetValues(values, idx, maxCountUpValues)
            idx += maxCountUpValues
            loops -= 1UI
        End While

        Dim remaining As UInteger = numberOfValues Mod maxCountUpValues
        SetValues(values, idx, remaining)
        If lastValueZero Then
            values(CInt(numberOfValues - 1UI)) = 0
        End If

        Return values
    End Function

    Private Sub SetValues(ByRef values As UInteger(), ByVal valueIndex As UInteger, ByVal numValues As UInteger)
        Dim value As UInteger = 0
        While value < numValues
            values(CInt(valueIndex)) = value
            valueIndex += 1UI
            value += 1UI
        End While
    End Sub

    Private Function GetMaxNumberOfValues(portWidthBits As UInteger) As UInteger
        If portWidthBits < 32 Then
            Return CUInt(1 << CInt(portWidthBits))
        Else
            Return &HFFFF_FFFFUI
        End If
    End Function
End Class
