﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferDO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports WriteBufferDO.ALLDAQ.Boards

Public Class AdqCommon
    Public Shared ReadOnly NotOpen As UInteger = AdqErrors.ADQ_ERROR_TYPE_FATAL Or AdqErrors.ADQ_ERROR_NOT_OPEN
    Public Shared ReadOnly DefaultRepeatBufferCount As UInteger = 1

    Public Shared Function GetFunctionErrorText(functionName As String, adqError As UInteger) As String
        Return String.Format("Function {0} returned Error number &H{1:X}", functionName, adqError)
    End Function

    Public Shared Function GetModuleTypeName(moduleType As UInteger) As String
        Return GetDictionaryValue(_moduleTypeNames, moduleType)
    End Function

    Private Shared Function GetDictionaryValue(dictionary As IDictionary(Of UInteger, String), key As UInteger) As String
        Dim stringValue As String = String.Empty
        Dim found As Boolean = dictionary.TryGetValue(key, stringValue)
        If (Not found) Then
            stringValue = String.Format("&H{0:X8}", key)
        End If
        Return stringValue
    End Function

    Private Shared ReadOnly _moduleTypeNames As Dictionary(Of UInteger, String) =
        New Dictionary(Of UInteger, String) From
        {
            {AdqConstants.ADQ_MODULE_TYPE_DIO, "DIO"},
            {AdqConstants.ADQ_MODULE_TYPE_DO, "DO"},
            {AdqConstants.ADQ_MODULE_TYPE_DI, "DI"},
            {AdqConstants.ADQ_MODULE_TYPE_CNTR, "Counter"},
            {AdqConstants.ADQ_MODULE_TYPE_EXTINT, "External Interrupt"},
            {AdqConstants.ADQ_MODULE_TYPE_AI, "AI"},
            {AdqConstants.ADQ_MODULE_TYPE_AO, "AO"},
            {AdqConstants.ADQ_MODULE_TYPE_I2C, "I2C"},
            {AdqConstants.ADQ_MODULE_TYPE_PWM, "PWM"},
            {AdqConstants.ADQ_MODULE_TYPE_FIN, "FIN"},
            {AdqConstants.ADQ_MODULE_TYPE_I2SO, "I2SO"}
        }

    Public Shared Function GetModelName(boardName As String) As String
        Dim posADQ As Integer = boardName.IndexOf("ADQ")
        If (posADQ < 0) Then
            posADQ = 0
        End If

        Dim posEnd As Integer = boardName.IndexOf(" ", posADQ)
        If (posEnd < 0) Then
            posEnd = boardName.Length
        End If

        Return boardName.Substring(posADQ, posEnd - posADQ)
    End Function

    Public Shared Function FormatFrequency(frequency As Double) As String
        Dim s As String
        If (frequency >= 1000000) Then
            s = String.Format("{0:0.######}MHz", frequency / 1000000)
        ElseIf (frequency >= 1000) Then
            s = String.Format("{0:0.######}kHz", frequency / 1000)
        Else
            s = String.Format("{0:0.########}Hz", frequency)
        End If
        Return s
    End Function

    Public Shared Function FormatOutputTime(seconds As Double) As String
        Dim s As String = String.Empty
        Try
            Dim ts As TimeSpan = TimeSpan.FromSeconds(seconds)
            s = String.Concat(IIf(ts.Days >= 1, String.Concat(ts.Days, " d "), ""),
                              IIf(ts.Hours >= 1, String.Concat(ts.Hours, " h "), ""),
                              IIf(ts.Minutes >= 1, String.Concat(ts.Minutes, " min "), ""),
                              IIf(ts.Seconds >= 1, String.Concat(ts.Seconds, " sec "), ""),
                              IIf(ts.Milliseconds >= 1, String.Concat(ts.Milliseconds, " ms"), ""))

            If s = String.Empty AndAlso seconds > 0 Then
                s = "< 1ms"
            End If
        Catch ovrflwEx As OverflowException
            s = String.Empty
        End Try
        Return s
    End Function
End Class
