//-----------------------------------------------------------------------------
// adqDevice.h - function prototypes for ALLDAQ driver system API
//      Copyright (c) ALLDAQ, a division of ALLNET GmbH
//      https://www.alldaq.com
//
// Purpose:
//      This file has defines for function prototypes
//      used by ALLDAQ measurement boards
//-----------------------------------------------------------------------------

#ifndef ADQDEVICEDLL_H
#define ADQDEVICEDLL_H

#include "adqDeviceConstants.h"

#include "adqDeviceErrors.h"    

#ifdef __cplusplus

extern "C"
{

#endif

    /*! \name	API routines
       
     */

    //@{

    unsigned int __stdcall adqGetInternalError(unsigned int& uiComponent, unsigned int& uiFile, unsigned int& uiLineNumber, unsigned int& uiType, unsigned int& uiErrorParameter1, unsigned int& uiErrorParameter2);

    unsigned int __stdcall adqGetNumberOfUsers(int& iNumberOfUsers);

    unsigned int __stdcall adqOpen(void);

    unsigned int __stdcall adqOpenEx(wchar_t** ppwcNameOrIPAddress);

    unsigned int __stdcall adqGetSystemInfo( SADQSystemInfo& sSystemInfo);

    unsigned int __stdcall adqQuery(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiID, unsigned int uiSubId, unsigned int uiBufferSize, unsigned char* pucBuffer, unsigned int& uiInfoSize);

    unsigned int __stdcall adqGetNumberOfBoards(unsigned int& uiNumberOfBoards);

    unsigned int __stdcall adqGetBoardInfo(unsigned int uiBoardIndex, SADQBoardInfo& sBoardInfo);

    unsigned int __stdcall adqGetBoardPCIInfo(unsigned int uiBoardIndex, SADQBoardPCIInfo& sBoardPCIInfo);

    unsigned int __stdcall adqResetBoard(unsigned int uiBoardIndex);

    unsigned int __stdcall adqGetBoardTTLVoltage(unsigned int uiBoardIndex, unsigned int& uiVoltage);

    unsigned int __stdcall adqSetBoardTTLVoltage(unsigned int uiBoardIndex, unsigned int uiVoltage);

    unsigned int __stdcall adqGetNumberOfModules(unsigned int uiBoardIndex, unsigned int& uiNumberOfModules);

    unsigned int __stdcall adqGetBoardSynchronStartInfo(unsigned int uiBoardIndex, SADQBoardSynchronStartInfo& sBoardSynchronInfo);

    unsigned int __stdcall adqConfigBoardSynchronStart(unsigned int uiBoardIndex, unsigned int uiGroupIndex, unsigned int uiModuleMap);

    unsigned int __stdcall adqBoardSynchronStart(unsigned int uiBoardIndex, unsigned int uiGroupIndex);

    unsigned int __stdcall adqGetModuleType(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int& uiModuleType);

    unsigned int __stdcall adqGetModuleInfo(unsigned int uiBoardIndex, unsigned int uiModuleIndex, SADQGeneralModuleInfo& sModuleInfo, unsigned int uiBufferSize, unsigned char* pucBuffer, unsigned int& uiInfoSize);

    unsigned int __stdcall adqResetModule(unsigned int uiBoardIndex, unsigned int uiModuleIndex);

    unsigned int __stdcall adqGetMaximumSampleFrequency(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiNumberOfChannels, double& dMaximumSampleFrequency);

    unsigned int __stdcall adqSetFrequency(unsigned int uiBoardIndex, unsigned int uiModuleIndex, double dNominalFrequency, double& dActualFrequency);

    unsigned int __stdcall adqGetHWTriggerInfo(unsigned int uiBoardIndex, unsigned int uiModuleIndex, SADQHWTriggerInfo& sHWTriggerInfo);

    unsigned int __stdcall adqEnableHWTrigger(unsigned int uiBoardIndex, unsigned int uiModuleIndex, SADQHWTriggerEnable& sHWTriggerEnable, bool bWaitForTrigger, unsigned int uiTimeoutMilliseconds);

    unsigned int __stdcall adqDisableHWTrigger(unsigned int uiBoardIndex, unsigned int uiModuleIndex);

    unsigned int __stdcall adqReadSingle(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int& uiValue);

    unsigned int __stdcall adqReadBuffer(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiNumberOfValuesRequired, unsigned int* puiValues, unsigned int& uiNumberOfValuesReturned);

    unsigned int __stdcall adqReadContinuousStart(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiNotifyAfterValues, unsigned __int64 ui64StopAfterValues, ADQ_READ_CONTINUOUS_CALLBACK adqReadContinuousCallback, void* pvReadContinuousUserContext);

    unsigned int __stdcall adqReadContinuous(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiNumberOfValuesRequired, unsigned int* puiValues, bool bWaitForData, unsigned int& uiNumberOfValuesReturned);

    unsigned int __stdcall adqReadContinuousStatus(unsigned int uiBoardIndex, unsigned int uiModuleIndex, bool bWaitForIdle, unsigned int& uiNumberOfValuesAvailable, bool& bRunning);

    unsigned int __stdcall adqWriteSingle(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiValue);

    unsigned int __stdcall adqWriteBuffer(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiBufferSizeValues, unsigned int* puiValues, unsigned int uiRepeatBuffer, unsigned int& uiNumberOfValuesWritten);

    unsigned int __stdcall adqWriteContinuousStart(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiBufferSizeValues, unsigned int* puiValues, unsigned int uiRepeatBuffer, bool bWaitForWrite, unsigned int uiNotifyAfterValues, unsigned int& uiNumberOfValuesWritten, ADQ_WRITE_CONTINUOUS_CALLBACK adqWriteContinuousCallback, void* pvWriteContinuousUserContext);

    unsigned int __stdcall adqWriteContinuous(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiBufferSizeValues, unsigned int* puiValues, unsigned int uiRepeatBuffer, bool bWaitForWrite, unsigned int& uiNumberOfValuesWritten);

    unsigned int __stdcall adqWriteContinuousStatus(unsigned int uiBoardIndex, unsigned int uiModuleIndex, bool bWaitForIdle, unsigned int& uiNumberOfValuesAcceptable, bool& bRunning);

    unsigned int __stdcall adqStop(unsigned int uiBoardIndex, unsigned int uiModuleIndex);
    
    unsigned int __stdcall adqGetIRQInfo(unsigned int uiBoardIndex, unsigned int uiModuleIndex, SADQIRQInfo& sIRQInfo);

    unsigned int __stdcall adqEnableIRQ(unsigned int uiBoardIndex, unsigned int uiModuleIndex, SADQIRQEnable& sIRQEnable, ADQ_IRQ_CALLBACK adqIRQCallback, unsigned int uiSADQIRQStatusID, void* pvIRQUserContext);

    unsigned int __stdcall adqWaitIRQ(unsigned int uiBoardIndex, unsigned int uiModuleIndex, SADQIRQStatus& sIRQStatus, unsigned int uiTimeoutMilliseconds);

    unsigned int __stdcall adqGetIRQCount(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int& uiIRQCount);

    unsigned int __stdcall adqDisableIRQ(unsigned int uiBoardIndex, unsigned int uiModuleIndex);

    unsigned int __stdcall adqDIOGetDirection(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int& uiDirection);

    unsigned int __stdcall adqDIOSetDirection(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiDirection);

    unsigned int __stdcall adqDIOGetFilterPicoseconds(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned __int64& ui64FilterPicoseconds);

    unsigned int __stdcall adqDIOSetFilterPicoseconds(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned __int64 ui64FilterPicoseconds);

    unsigned int __stdcall adqCNTRConfig(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiClockSource, unsigned int uiCntrMode, unsigned int uiStartValue);

    unsigned int __stdcall adqCNTRConfigEx(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiCNTRConfigSize, unsigned char* pucCNTRConfig);

    unsigned int __stdcall adqCNTREnable(unsigned int uiBoardIndex, unsigned int uiModuleIndex);

    unsigned int __stdcall adqCNTRDisable(unsigned int uiBoardIndex, unsigned int uiModuleIndex);

    unsigned int __stdcall adqAISetChannelAndRange(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiChannelIndex, unsigned int uiRangeIndex);

    unsigned int __stdcall adqAIGetChannelHardwareRange(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiChannelIndex, unsigned int& uiRangeIndex);

    unsigned int __stdcall adqAISetChannelList(unsigned int uiBoardIndex, unsigned int uiModuleIndex, SADQAIChannel* parrsChannels, unsigned int uiNumberOfChannels);

    unsigned int __stdcall adqAISetChannelListAndFrequency(unsigned int uiBoardIndex, unsigned int uiModuleIndex, SADQAIChannel* parrsChannels, unsigned int uiNumberOfChannels, double dNominalFrequency, double& dActualFrequency);

    unsigned int __stdcall adqAIReadSingle(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiFrameLength, unsigned int* puiFrame);

    unsigned int __stdcall adqAIReadBuffer(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiFrameLength, unsigned int uiNumberOfFramesRequired, unsigned int* puiFrames, unsigned int& uiNumberOfFramesReturned);

    unsigned int __stdcall adqAIReadContinuousStart(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiNotifyAfterFrames, unsigned __int64 ui64StopAfterFrames, ADQ_AI_READ_CONTINUOUS_CALLBACK adqAIReadContinuousCallback, void* pvAIReadContinuousUserContext);

    unsigned int __stdcall adqAIReadContinuous(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiFrameLength, unsigned int uiNumberOfFramesRequired, unsigned int* puiFrames, bool bWaitForData, unsigned int& uiNumberOfFramesReturned);

    unsigned int __stdcall adqAIReadContinuousStatus(unsigned int uiBoardIndex, unsigned int uiModuleIndex, bool bWaitForIdle, unsigned int& uiNumberOfFramesAvailable, bool& bRunning);

    unsigned int __stdcall adqAISetCalibrationZero(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiChannelIndex, unsigned int uiRangeIndex);

    unsigned int __stdcall adqAnalogGetSelectedCalibration(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int& uiCalibrationID);

    unsigned int __stdcall adqAnalogGetNumberOfUserCalibrations(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int& uiNumberOfUserCalibrations);

    unsigned int __stdcall adqAnalogSelectCalibration(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiCalibrationID);

    unsigned int __stdcall adqAnalogSetTemporaryCalibration(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiChannelIndex, unsigned int uiRangeIndex, double dNominalPhysicalValue1, double dActualPhysicalValue1, double dNominalPhysicalValue2, double dActualPhysicalValue2);

    unsigned int __stdcall adqAnalogGetCalibrationFactorAndConstant(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiChannelIndex, unsigned int uiRangeIndex, unsigned int uiCalibrationID, double& dCalibrationFactor, double& dCalibrationConstant);

    unsigned int __stdcall adqI2CSetSlaveAddress(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiSlaveAddress);

    unsigned int __stdcall adqI2CRead(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiNumberOfBytesRequired, unsigned char* pucData, unsigned int& uiNumberOfBytesReturned);

    unsigned int __stdcall adqI2CWrite(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiNumberOfBytesToWrite, unsigned char* pucData, unsigned int& uiNumberOfBytesWritten);

    unsigned int __stdcall adqI2CWriteThenRead(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiNumberOfBytesToWrite, unsigned char* pucWriteData, unsigned int& uiNumberOfBytesWritten, unsigned int uiNumberOfBytesRequired, unsigned char* pucReadData, unsigned int& uiNumberOfBytesReturned);

    unsigned int __stdcall adqPWMGetMaximumFrequency(unsigned int uiBoardIndex, unsigned int uiModuleIndex, double& dMaximumFrequency);

    unsigned int __stdcall adqPWMConfig(unsigned int uiBoardIndex, unsigned int uiModuleIndex, double dNominalFrequency, double dNominalOnTime, bool bInvert, bool bHardwareTrigger, double& dActualFrequency, double& dActualOnTime);

    unsigned int __stdcall adqPWMEnable(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiTimeoutMilliseconds);

    unsigned int __stdcall adqPWMDisable(unsigned int uiBoardIndex, unsigned int uiModuleIndex);

    unsigned int __stdcall adqFINRead(unsigned int uiBoardIndex, unsigned int uiModuleIndex, double& dFrequency, double& dOnTime, unsigned int uiTimeoutMilliseconds);

    unsigned int __stdcall adqI2SOSineConfig(unsigned int uiBoardIndex, unsigned int uiModuleIndex, double dNominalOutputFrequency, 
                                                unsigned int uiSignalLeftPrecisionBits, bool bSignalLeftTwosComplement, double dSignalLeftSignalFrequency, double dSignalLeftAmplitude, 
                                                    unsigned int uiSignalRightPrecisionBits, bool bSignalRightTwosComplement, double dSignalRightSignalFrequency, double dSignalRightAmplitude, 
                                                                                                                                                                        double& dActualOutputFrequency);

    unsigned int __stdcall adqI2SOSineStart(unsigned int uiBoardIndex, unsigned int uiModuleIndex);

    unsigned int __stdcall adqI2SOSineModify(unsigned int uiBoardIndex, unsigned int uiModuleIndex, double dSignalLeftSignalFrequency, double dSignalLeftAmplitude, double dSignalRightSignalFrequency, double dSignalRightAmplitude);

    unsigned int __stdcall adqI2SOSineStop(unsigned int uiBoardIndex, unsigned int uiModuleIndex);

    unsigned int __stdcall adqClose(void);

    //@}

#ifdef __cplusplus

}

#endif


#endif