#include <windows.h>

#include <stdio.h>
#include <stdlib.h>

#include <tchar.h>

#include <conio.h>

#include "..\\adqDevice.h"

#ifdef WIN64

#pragma comment(lib, "..\\adqDevice_64.lib")

#else

#pragma comment(lib, "..\\adqDevice.lib")

#endif

struct SI2CModule
{
    unsigned int    m_uiBoardIndex;

    unsigned int    m_uiModuleIndex;

    unsigned int    m_uiSpeedBitsPerSecond;

    bool            m_bSupports7BitAddresses;

    bool            m_bSupports10BitAddresses;
};

int _tmain(int argc, _TCHAR* argv[])
{
   // Increase the size of the console buffer
    
    HANDLE  h_console = GetStdHandle(STD_OUTPUT_HANDLE);

    COORD coord;

    coord.X = 160;
    
    coord.Y = 9999;
    
    SetConsoleScreenBufferSize(h_console, coord);

    bool b_terminate = false;

    bool b_remote = false;

    wchar_t wsz_remote_name_or_IP_address[128];

    while(true)
    {
        printf( "1. Open local\n"
                "2. Open remote\n"
                "\n"
                "Your choice: 1 or 2 (RETURN to exit): ");

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            b_terminate = true;
            
            break;
        }

        unsigned int ui_choice = 0;

        if( sscanf_s(&sz_response[0], "%u", &ui_choice) != 1)
        {
            continue;
        }

        if(ui_choice == 1)
        {
            break;
        }

        if(ui_choice == 2)
        {
            b_remote = true;

            printf("Name or IP-Address (RETURN to exit): ");

            _getws_s(&wsz_remote_name_or_IP_address[0], 128);

            printf("\n\n");

            if( wsz_remote_name_or_IP_address[0] == (wchar_t)0 )
            {
                b_terminate = true;
            }

            break;
        }
    }

    if(b_terminate)
    {
        printf("Press any key to exit.\n");

        _getch();

        exit(-1);
    }

    unsigned int ui_adq_error;
    
    if(b_remote)
    {
        wchar_t* pwc_user_name_or_address[2] = {&wsz_remote_name_or_IP_address[0],
                                                NULL                                };

        ui_adq_error = adqOpenEx(pwc_user_name_or_address);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("!!!! Error - adqOpenEx returned error code 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }
    else
    {
        ui_adq_error = adqOpen();

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("!!!! Error - adqOpen returned error code 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }

    unsigned int ui_number_of_boards;
    
    ui_adq_error = adqGetNumberOfBoards(ui_number_of_boards);

    if(ui_adq_error != ADQ_ERROR_NONE)
    {
        printf("!!!! Error - adqGetNumberOfBoards returned error code 0x%08X\n\n", ui_adq_error);

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    if(ui_number_of_boards == 0)
    {
        printf("No ADQ boards found!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    printf("%3u ADQ Boards found\n\n", ui_number_of_boards);

    // Count the number of I2C modules

    unsigned int ui_number_of_i2c_modules = 0;

    for(unsigned int ui_index_board = 0; ui_index_board < ui_number_of_boards; ui_index_board++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_index_board, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("!!!! Error -     Board index: %3u adqGetNumberOfModules returned error code 0x%08X\n\n", ui_index_board, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_index_module = 0; ui_index_module < ui_number_of_modules; ui_index_module++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_index_board, ui_index_module, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("!!!! Error -         Board index: %3u Module index: %3u adqGetModuleType returned error code 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            if(ui_module_type == ADQ_MODULE_TYPE_I2C)
            {
                ++ui_number_of_i2c_modules;
            }
        }
    }

    if(ui_number_of_i2c_modules == 0)
    {
        printf("No I2C modules found!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    unsigned int ui_max_number_of_i2c_modules = ui_number_of_i2c_modules;

    SI2CModule* arr_i2c_modules = new SI2CModule[ui_max_number_of_i2c_modules];

    ui_number_of_i2c_modules = 0;

    for(unsigned int ui_index_board = 0; ui_index_board < ui_number_of_boards; ui_index_board++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_index_board, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("!!!! Error -     Board index: %3u adqGetNumberOfModules returned error code 0x%08X\n\n", ui_index_board, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_index_module = 0; ui_index_module < ui_number_of_modules; ui_index_module++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_index_board, ui_index_module, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("!!!! Error -         Board index: %3u Module index: %3u adqGetModuleType returned error code 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            SADQGeneralModuleInfo s_adq_general_module_info;

            s_adq_general_module_info.m_uiSize = sizeof(SADQGeneralModuleInfo);

            s_adq_general_module_info.m_uiID = SADQGeneralModuleInfo_ID;

            if(ui_module_type == ADQ_MODULE_TYPE_I2C)
            {
                arr_i2c_modules[ui_number_of_i2c_modules].m_uiBoardIndex = ui_index_board;

                arr_i2c_modules[ui_number_of_i2c_modules].m_uiModuleIndex = ui_index_module;

                unsigned int ui_info_size;

                SADQI2CModuleInfo s_adq_i2c_module_info;

                s_adq_i2c_module_info.m_uiSize = sizeof(SADQI2CModuleInfo);

                s_adq_i2c_module_info.m_uiID = SADQDIModuleInfo_ID;

                ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQI2CModuleInfo), (unsigned char*)&s_adq_i2c_module_info, ui_info_size);

                if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("        Board index: %3u Module index (I2C module): %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board + 1, ui_index_module + 1, ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    continue;
                }

                arr_i2c_modules[ui_number_of_i2c_modules].m_uiSpeedBitsPerSecond = s_adq_i2c_module_info.m_uiSpeedBitsPerSecond;

                arr_i2c_modules[ui_number_of_i2c_modules].m_bSupports7BitAddresses = s_adq_i2c_module_info.m_bSupports7BitAddresses;

                arr_i2c_modules[ui_number_of_i2c_modules].m_bSupports10BitAddresses = s_adq_i2c_module_info.m_bSupports10BitAddresses;

                ++ui_number_of_i2c_modules;
            }
        }
    }

    while(true)
    {
        printf("Please choose a module:\n\n");

        unsigned int ui_index_i2c_module;

        for(ui_index_i2c_module = 0; ui_index_i2c_module < ui_number_of_i2c_modules; ui_index_i2c_module++)
        {
            printf("%2u.  Board index: %2d  Module index: %-2d - Speed: %u Bits per Second, Supports 7-Bit Addresses: %s, Supports 10-Bit Addresses: %s\n",
                   ui_index_i2c_module + 1,
                   arr_i2c_modules[ui_index_i2c_module].m_uiBoardIndex,
                   arr_i2c_modules[ui_index_i2c_module].m_uiModuleIndex,
                   arr_i2c_modules[ui_index_i2c_module].m_uiSpeedBitsPerSecond,
                   arr_i2c_modules[ui_index_i2c_module].m_bSupports7BitAddresses ? "Yes" : " No",
                   arr_i2c_modules[ui_index_i2c_module].m_bSupports10BitAddresses ? "Yes" : " No" );
        }

        printf("\n");

        printf("Your choice 1 - %d (RETURN to exit): ",
               ui_number_of_i2c_modules             );

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            break;
        }

        if( sscanf_s(&sz_response[0], "%u", &ui_index_i2c_module) != 1)
        {
            continue;
        }

        if( (ui_index_i2c_module < 1)||(ui_index_i2c_module > ui_number_of_i2c_modules) )
        {
            continue;
        }

        --ui_index_i2c_module;

        for( ; ; )
        {
            printf("(S)et slave address\n"
                   "(R)ead byte\n"
                   "(W)rite byte\n\n"
                   "Your choice S, R or W (ESC to cancel): " );

            char ch_user = _getch();

            printf("%c\n\n", ch_user);

            if( (ch_user == 'S') ||(ch_user == 's') )
            {
                for( ; ; )
                {
                    printf( "Slave Address: 0x");

                    char sz_response[8];

                    gets_s(&sz_response[0], 8);

                    printf("\n\n");

                    if(sz_response[0] == 0)
                    {
                        continue;
                    }

                    unsigned int ui_slave_address;

                    if( sscanf_s(&sz_response[0], "%x", &ui_slave_address) != 1 )
                    {
                        continue;
                    }

                    ui_adq_error = adqI2CSetSlaveAddress(   arr_i2c_modules[ui_index_i2c_module].m_uiBoardIndex,
                                                            arr_i2c_modules[ui_index_i2c_module].m_uiModuleIndex,
                                                            ui_slave_address                                    );


                    if(ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("!!!! Error - adqI2CSetSlaveAddress returned error code 0x%08X\n\n", ui_adq_error);

                        break;
                    }

                    printf( "Slave Address set to: 0x%08X (%u)\n\n",
                            ui_slave_address, ui_slave_address      );

                    break;
                }

                printf("Press any key to continue.\n");

                _getch();
            }
            else if( (ch_user == 'R') ||(ch_user == 'r') )
            {
                unsigned int ui_value;
                
                ui_adq_error = adqReadSingle(   arr_i2c_modules[ui_index_i2c_module].m_uiBoardIndex,
                                                arr_i2c_modules[ui_index_i2c_module].m_uiModuleIndex,
                                                ui_value                                                );

                if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("!!!! Error - adqReadSingle returned error code 0x%08X\n\n", ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    break;
                }

                printf( "Read 0x%02X (%u) from Board index %u Module index %u\n\n",
                        (unsigned char)(ui_value & 0x000000FF), 
                        (unsigned char)(ui_value & 0x000000FF),
                        arr_i2c_modules[ui_index_i2c_module].m_uiBoardIndex,
                        arr_i2c_modules[ui_index_i2c_module].m_uiModuleIndex           );

                printf("Press any key to continue.\n");

                _getch();
            }
            else if( (ch_user == 'W') ||(ch_user == 'w') )
            {
                for( ; ; )
                {
                    printf("Write value to Board index %u Module index %u: 0x",
                            arr_i2c_modules[ui_index_i2c_module].m_uiBoardIndex,
                            arr_i2c_modules[ui_index_i2c_module].m_uiModuleIndex    );

                    char sz_response[8];

                    gets_s(&sz_response[0], 8);

                    printf("\n\n");

                    if(sz_response[0] == 0)
                    {
                        continue;
                    }

                    unsigned int ui_value;
                
                    if( sscanf_s(&sz_response[0], "%x", &ui_value) != 1 )
                    {
                        continue;
                    }

                    ui_adq_error = adqWriteSingle(   arr_i2c_modules[ui_index_i2c_module].m_uiBoardIndex,
                                                    arr_i2c_modules[ui_index_i2c_module].m_uiModuleIndex,
                                                    ui_value                                                );

                    if(ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("!!!! Error - adqWriteSingle returned error code 0x%08X\n\n", ui_adq_error);

                        break;
                    }

                    printf( "Wrote 0x%02X (%u) to Board index %u Module index %u\n\n",
                            (unsigned char)(ui_value & 0x000000FF), 
                            (unsigned char)(ui_value & 0x000000FF),
                            arr_i2c_modules[ui_index_i2c_module].m_uiBoardIndex,
                            arr_i2c_modules[ui_index_i2c_module].m_uiModuleIndex           );

                    break;
                }

                printf("Press any key to continue.\n");

                _getch();
            }
            else if(ch_user == 27)
            {
                break;
            }
        }

        printf("Press any key to continue\n\n");

        _getch();
    }

    delete []arr_i2c_modules;

    adqClose();

    printf("\n\n");

    printf("Press any key to terminate.\n");

    _getch();

    return 0;
}

