#include <windows.h>

#include <stdio.h>
#include <stdlib.h>

#include <tchar.h>

#include <conio.h>

#include "..\\adqDevice.h"

#ifdef WIN64

#pragma comment(lib, "..\\adqDevice_64.lib")

#else

#pragma comment(lib, "..\\adqDevice.lib")

#endif

struct SPWMModule
{
    unsigned int    m_uiBoardIndex;

    unsigned int    m_uiModuleIndex;

    double          m_dBaseFrequency;

    unsigned int    m_uiMinimumDivider;
};

int _tmain(int argc, _TCHAR* argv[])
{
   // Increase the size of the console buffer
    
    HANDLE  h_console = GetStdHandle(STD_OUTPUT_HANDLE);

    COORD coord;

    coord.X = 160;
    
    coord.Y = 9999;
    
    SetConsoleScreenBufferSize(h_console, coord);

    bool b_terminate = false;

    bool b_remote = false;

    wchar_t wsz_remote_name_or_IP_address[128];

    while(true)
    {
        printf( "1. Open local\n"
                "2. Open remote\n"
                "\n"
                "Your choice: 1 or 2 (RETURN to exit): ");

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            b_terminate = true;
            
            break;
        }

        unsigned int ui_choice = 0;

        if( sscanf_s(&sz_response[0], "%u", &ui_choice) != 1)
        {
            continue;
        }

        if(ui_choice == 1)
        {
            break;
        }

        if(ui_choice == 2)
        {
            b_remote = true;

            printf("Name or IP-Address (RETURN to exit): ");

            _getws_s(&wsz_remote_name_or_IP_address[0], 128);

            printf("\n\n");

            if( wsz_remote_name_or_IP_address[0] == (wchar_t)0 )
            {
                b_terminate = true;
            }

            break;
        }
    }

    if(b_terminate)
    {
        printf("Press any key to exit.\n");

        _getch();

        exit(-1);
    }

    unsigned int ui_adq_error;
    
    if(b_remote)
    {
        wchar_t* pwc_user_name_or_address[2] = {&wsz_remote_name_or_IP_address[0],
                                                NULL                                };

        ui_adq_error = adqOpenEx(pwc_user_name_or_address);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("!!!! Error - adqOpenEx returned error code 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }
    else
    {
        ui_adq_error = adqOpen();

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("!!!! Error - adqOpen returned error code 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }

    unsigned int ui_number_of_boards;
    
    ui_adq_error = adqGetNumberOfBoards(ui_number_of_boards);

    if(ui_adq_error != ADQ_ERROR_NONE)
    {
        printf("!!!! Error - adqGetNumberOfBoards returned error code 0x%08X\n\n", ui_adq_error);

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    if(ui_number_of_boards == 0)
    {
        printf("No ADQ boards found!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    printf("%3u ADQ Boards found\n\n", ui_number_of_boards);

    // Count the number of PWM modules

    unsigned int ui_number_of_pwm_modules = 0;

    for(unsigned int ui_index_board = 0; ui_index_board < ui_number_of_boards; ui_index_board++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_index_board, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("!!!! Error -     Board index: %3u adqGetNumberOfModules returned error code 0x%08X\n\n", ui_index_board, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_index_module = 0; ui_index_module < ui_number_of_modules; ui_index_module++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_index_board, ui_index_module, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("!!!! Error -         Board index: %3u Module index: %3u adqGetModuleType returned error code 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            if(ui_module_type == ADQ_MODULE_TYPE_PWM)
            {
                ++ui_number_of_pwm_modules;
            }
        }
    }

    if(ui_number_of_pwm_modules == 0)
    {
        printf("No PWM modules found!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    unsigned int ui_max_number_of_pwm_modules = ui_number_of_pwm_modules;

    SPWMModule* arr_pwm_modules = new SPWMModule[ui_max_number_of_pwm_modules];

    ui_number_of_pwm_modules = 0;

    for(unsigned int ui_index_board = 0; ui_index_board < ui_number_of_boards; ui_index_board++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_index_board, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("!!!! Error -     Board index: %3u adqGetNumberOfModules returned error code 0x%08X\n\n", ui_index_board, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_index_module = 0; ui_index_module < ui_number_of_modules; ui_index_module++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_index_board, ui_index_module, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("!!!! Error -         Board index: %3u Module index: %3u adqGetModuleType returned error code 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            SADQGeneralModuleInfo s_adq_general_module_info;

            s_adq_general_module_info.m_uiSize = sizeof(SADQGeneralModuleInfo);

            s_adq_general_module_info.m_uiID = SADQGeneralModuleInfo_ID;

            if(ui_module_type == ADQ_MODULE_TYPE_PWM)
            {
                arr_pwm_modules[ui_number_of_pwm_modules].m_uiBoardIndex = ui_index_board;

                arr_pwm_modules[ui_number_of_pwm_modules].m_uiModuleIndex = ui_index_module;

                unsigned int ui_info_size;

                SADQPWMModuleInfo s_adq_pwm_module_info;

                s_adq_pwm_module_info.m_uiSize = sizeof(SADQPWMModuleInfo);

                s_adq_pwm_module_info.m_uiID = SADQPWMModuleInfo_ID;

                ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQPWMModuleInfo), (unsigned char*)&s_adq_pwm_module_info, ui_info_size);

                if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("        Board index: %3u Module index (I2C module): %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board + 1, ui_index_module + 1, ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    continue;
                }

                arr_pwm_modules[ui_number_of_pwm_modules].m_dBaseFrequency = s_adq_pwm_module_info.m_dBaseFrequencyHz;

                arr_pwm_modules[ui_number_of_pwm_modules].m_uiMinimumDivider = s_adq_pwm_module_info.m_uiMinimumDivider;

                ++ui_number_of_pwm_modules;
            }
        }
    }

    while(true)
    {
        printf("Please choose a module:\n\n");

        unsigned int ui_index_pwm_module;

        for(ui_index_pwm_module = 0; ui_index_pwm_module < ui_number_of_pwm_modules; ui_index_pwm_module++)
        {
            printf("%2u.  Board index: %2d  Module index: %-2d - Base frequency: %14.5f Hz, Minimum divider: %u\n",
                   ui_index_pwm_module + 1,
                   arr_pwm_modules[ui_index_pwm_module].m_uiBoardIndex,
                   arr_pwm_modules[ui_index_pwm_module].m_uiModuleIndex,
                   arr_pwm_modules[ui_index_pwm_module].m_dBaseFrequency,
                   arr_pwm_modules[ui_index_pwm_module].m_uiMinimumDivider                                          );
        }

        printf("\n");

        printf("Your choice 1 - %d (RETURN to exit): ",
               ui_number_of_pwm_modules             );

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            break;
        }

        if( sscanf_s(&sz_response[0], "%u", &ui_index_pwm_module) != 1)
        {
            continue;
        }

        if( (ui_index_pwm_module < 1)||(ui_index_pwm_module > ui_number_of_pwm_modules) )
        {
            continue;
        }

        --ui_index_pwm_module;

        for( ; ; )
        {
            printf("(G)et maximum frequency\n"
                   "(C)onfig\n"
                   "(E)nable\n"
                   "(D)isable\n\n"
                   "Your choice G, C, E or D (ESC to cancel): " );

            char ch_user = _getch();

            printf("%c\n\n", ch_user);

            if( (ch_user == 'G') ||(ch_user == 'g') )
            {
                double d_maximum_frequency;
                
                ui_adq_error = adqPWMGetMaximumFrequency(   arr_pwm_modules[ui_index_pwm_module].m_uiBoardIndex,
                                                            arr_pwm_modules[ui_index_pwm_module].m_uiModuleIndex,
                                                            d_maximum_frequency                                 );

                if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("!!!! Error - adqPWMGetMaximumFrequency returned error code 0x%08X\n\n", ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    break;
                }

                printf( "Board index %u Module index %u - Maximum PWM frequency is: %14.5f Hz\n\n",
                        arr_pwm_modules[ui_index_pwm_module].m_uiBoardIndex,
                        arr_pwm_modules[ui_index_pwm_module].m_uiModuleIndex,
                        d_maximum_frequency                                                     );

                printf("Press any key to continue.\n");

                _getch();
            }
            else if( (ch_user == 'C') ||(ch_user == 'c') )
            {
                double d_nominal_frequency;
                
                for( ; ; )
                {
                    printf( "Nominal Frequency (Hz): ");

                    char sz_response[128];

                    gets_s(&sz_response[0], 128);

                    printf("\n\n");

                    if(sz_response[0] == 0)
                    {
                        continue;
                    }

                    if( sscanf_s(&sz_response[0], "%lf", &d_nominal_frequency) != 1 )
                    {
                        continue;
                    }

                    break;
                }

                double d_nominal_on_time;
                
                for( ; ; )
                {
                    printf( "Nominal on time (Seconds): ");

                    char sz_response[128];

                    gets_s(&sz_response[0], 128);

                    printf("\n\n");

                    if(sz_response[0] == 0)
                    {
                        continue;
                    }

                    if( sscanf_s(&sz_response[0], "%lf", &d_nominal_on_time) != 1 )
                    {
                        continue;
                    }

                    break;
                }

                bool b_invert = false;

                for( ; ; )
                {
                    printf("Invert Signal?\n"
                           "(N)o \n"
                           "(Y)es\n"
                           "Your choice N or Y: ");

                    char ch_response = _getch();

                    printf("%c\n\n", ch_response);

                    if( (ch_response == 'N')||(ch_response == 'n') )
                    {
                        b_invert = false;

                        break;
                    }
                    else if( (ch_response == 'Y')||(ch_response == 'y') )
                    {
                        b_invert = true;

                        break;
                    }
                }

                bool b_hardware_trigger = false;

                for( ; ; )
                {
                    printf("Hardware trigger?\n"
                           "(N)o \n"
                           "(Y)es\n"
                           "Your choice N or Y: ");

                    char ch_response = _getch();

                    printf("%c\n\n", ch_response);

                    if( (ch_response == 'N')||(ch_response == 'n') )
                    {
                        b_hardware_trigger = false;

                        break;
                    }
                    else if( (ch_response == 'Y')||(ch_response == 'y') )
                    {
                        b_hardware_trigger = true;

                        break;
                    }
                }

                double d_actual_frequency;

                double d_actual_on_time;
                
                ui_adq_error = adqPWMConfig(arr_pwm_modules[ui_index_pwm_module].m_uiBoardIndex,
                                            arr_pwm_modules[ui_index_pwm_module].m_uiModuleIndex,
                                            d_nominal_frequency,
                                            d_nominal_on_time,
                                            b_invert,
                                            b_hardware_trigger,
                                            d_actual_frequency, 
                                            d_actual_on_time                                    );

                if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("!!!! Error - adqPWMConfig returned error code 0x%08X\n\n", ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    break;
                }

                printf( "Board index %u Module index %u\n"
                        "    Called adqPWMConfig with nominal frequency: %f Hz and nominal on time: %f seconds\n"
                        "                             actual frequency;: %f Hz, actual on time: %f seconds\n\n",
                        arr_pwm_modules[ui_index_pwm_module].m_uiBoardIndex,
                        arr_pwm_modules[ui_index_pwm_module].m_uiModuleIndex,
                        d_nominal_frequency,
                        d_nominal_on_time,
                        d_actual_frequency,
                        d_actual_on_time                                                                        );

                printf("Press any key to continue.\n");

                _getch();
            }
            else if( (ch_user == 'E') ||(ch_user == 'e') )
            {
                unsigned int ui_time_out_milliseconds;
                
                for(; ;)
                {
                    printf("Time-out in milliseconds (RETURN to cancel): ");

                    char sz_response[32];

                    gets_s(&sz_response[0], 32);

                    printf("\n\n");

                    if( sscanf_s(&sz_response[0], "%u", &ui_time_out_milliseconds) != 1 )
                    {
                        continue;
                    }

                    break;
                }

                ui_adq_error = adqPWMEnable( arr_pwm_modules[ui_index_pwm_module].m_uiBoardIndex,
                                                arr_pwm_modules[ui_index_pwm_module].m_uiModuleIndex,
                                                ui_time_out_milliseconds                                );

                if(ui_adq_error == ADQ_ERROR_TIMEOUT)
                {
                    printf("!!!! Error - adqPWMEnable timed out\n\n");

                    printf("Press any key to continue.\n");

                    _getch();

                    break;
                }
                else if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("!!!! Error - adqPWMEnable returned error code 0x%08X\n\n", ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    break;
                }

                printf( "PWM Enabled\n\n");

                printf("Press any key to continue.\n");

                _getch();
            }
            else if( (ch_user == 'D') ||(ch_user == 'd') )
            {
                ui_adq_error = adqPWMDisable(  arr_pwm_modules[ui_index_pwm_module].m_uiBoardIndex,
                                                arr_pwm_modules[ui_index_pwm_module].m_uiModuleIndex    );

                if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("!!!! Error - adqPWMDisable returned error code 0x%08X\n\n", ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    break;
                }

                printf( "PWM Disabled\n\n");

                printf("Press any key to continue.\n");

                _getch();
            }
            else if(ch_user == 27)
            {
                break;
            }
        }

        printf("Press any key to continue\n\n");

        _getch();
    }

    delete []arr_pwm_modules;

    adqClose();

    printf("\n\n");

    printf("Press any key to terminate.\n");

    _getch();

    return 0;
}

