//-----------------------------------------------------------------------------
// adqDeviceLoadDll.h - function prototypes for ALLDAQ driver system API
//      Copyright (c) ALLDAQ, a division of ALLNET GmbH
//      https://www.alldaq.com
//
// Purpose:
//      This file has typedefs and function prototypes for 
//      explicitly linking to the dll using GetProcAddress().
//      Used by ALLDAQ measurement boards.
//
// Note:
//      Import library 32bit: adqDevice.dll
//                     64bit: adqDevice_64.dll 
//-----------------------------------------------------------------------------

#ifndef ADQDEVICE_LOAD_DLL_H
#define ADQDEVICE_LOAD_DLL_H

#include "adqDeviceConstants.h"
#include "adqDeviceErrors.h"


#ifdef WIN64
#define ADQ_DEVICE_DLL "adqDevice_64.dll"
#else                       
#define ADQ_DEVICE_DLL "adqDevice.dll"
#endif


//-----------------------------------------------------------------------------
// load library and free library functions
//-----------------------------------------------------------------------------
unsigned int adqDeviceLoadLibrary(void);
unsigned int adqDeviceFreeLibrary(void);

//-----------------------------------------------------------------------------
// typedefs for the exported API-function pointers
//-----------------------------------------------------------------------------
typedef unsigned int(__stdcall *fptr_adqGetInternalError)(
    unsigned int& uiComponent,
    unsigned int& uiFile,
    unsigned int& uiLineNumber,
    unsigned int& uiType,
    unsigned int& uiErrorParameter1,
    unsigned int& uiErrorParameter2);

typedef unsigned int (__stdcall *fptr_adqOpen)(void);

typedef unsigned int (__stdcall *fptr_adqOpenEx)(
    wchar_t** ppwcNameOrIPAddress);

typedef unsigned int (__stdcall *fptr_adqGetNumberOfUsers)(
    unsigned int& uiNumberOfUsers);

typedef unsigned int (__stdcall *fptr_adqGetSystemInfo)(
    SADQSystemInfo& sSystemInfo);

typedef unsigned int (__stdcall *fptr_adqQuery)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiID,
    unsigned int uiSubId,
    unsigned int uiBufferSize,
    unsigned char* pucBuffer,
    unsigned int& uiInfoSize);

typedef unsigned int (__stdcall *fptr_adqGetNumberOfBoards)(
    unsigned int& uiNumberOfBoards);

typedef unsigned int (__stdcall *fptr_adqGetBoardInfo)(
    unsigned int uiBoardIndex,
    SADQBoardInfo& sBoardInfo);

typedef unsigned int (__stdcall *fptr_adqGetBoardPCIInfo)(
    unsigned int uiBoardIndex, 
    SADQBoardPCIInfo& sBoardPCIInfo);    
    
typedef unsigned int (__stdcall *fptr_adqResetBoard)(
    unsigned int uiBoardIndex);

typedef unsigned int (__stdcall *fptr_adqGetBoardTTLVoltage)(
    unsigned int uiBoardIndex, 
    unsigned int& uiVoltage);    

typedef unsigned int (__stdcall *fptr_adqSetBoardTTLVoltage)(
    unsigned int uiBoardIndex, 
    unsigned int uiVoltage);    
    
typedef unsigned int (__stdcall *fptr_adqGetNumberOfModules)(
    unsigned int uiBoardIndex,
    unsigned int& uiNumberOfModules);

typedef unsigned int(__stdcall *fptr_adqGetModuleType)(
	unsigned int uiBoardIndex,
	unsigned int uiModuleIndex,
	unsigned int& uiModuleType);

typedef unsigned int(__stdcall *fptr_adqGetBoardSynchronStartInfo(
	unsigned int uiBoardIndex, 
	SADQBoardSynchronStartInfo& sBoardSynchronInfo);

typedef unsigned int(__stdcall *fptr_adqConfigBoardSynchronStart(
	unsigned int uiBoardIndex, 
	unsigned int uiGroupIndex, 
	unsigned int uiModuleMap);

typedef unsigned int(__stdcall *fptr_adqBoardSynchronStart(
	unsigned int uiBoardIndex, 
	unsigned int uiGroupIndex);

typedef unsigned int (__stdcall *fptr_adqGetModuleType)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int& uiModuleType);

typedef unsigned int (__stdcall *fptr_adqGetModuleInfo)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    SADQGeneralModuleInfo& sModuleInfo,
    unsigned int uiBufferSize,
    unsigned char* pucBuffer,
    unsigned int& uiInfoSize);

typedef unsigned int (__stdcall *fptr_adqResetModule)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex);

typedef unsigned int (__stdcall *fptr_adqGetMaximumSampleFrequency)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiNumberOfChannels, 
    double& dMaximumSampleFrequency);
    
typedef unsigned int (__stdcall *fptr_adqSetFrequency)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    double dNominalFrequency,
    double& dActualFrequency);

typedef unsigned int (__stdcall *fptr_adqGetHWTriggerInfo)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    SADQHWTriggerInfo& sHWTriggerInfo);

typedef unsigned int (__stdcall *fptr_adqEnableHWTrigger)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    SADQHWTriggerEnable& sHWTriggerEnable, 
    bool bWaitForTrigger, 
    unsigned int uiTimeoutMilliseconds);

typedef unsigned int(__stdcall *fptr_adqDisableHWTrigger)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex);

typedef unsigned int (__stdcall *fptr_adqReadSingle)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int& uiValue);

typedef unsigned int (__stdcall *fptr_adqReadBuffer)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiNumberOfValuesRequired,
    unsigned int* puiValues,
    unsigned int& uiNumberOfValuesReturned);

typedef unsigned int (__stdcall *fptr_adqReadContinuousStart)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiNotifyAfterValues,
    unsigned __int64 ui64StopAfterValues,
    ADQ_READ_CONTINUOUS_CALLBACK adqReadContinuousCallback,
    void* pvReadContinuousUserContext);

typedef unsigned int (__stdcall *fptr_adqReadContinuous)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiNumberOfValuesRequired,
    unsigned int* puiValues,
    bool bWaitForData,
    unsigned int& uiNumberOfValuesReturned);

typedef unsigned int (__stdcall *fptr_adqReadContinuousStatus)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    bool bWaitForIdle,
    unsigned int& uiNumberOfValuesAvailable,
    bool& bRunning);

typedef unsigned int (__stdcall *fptr_adqWriteSingle)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiValue);

typedef unsigned int (__stdcall *fptr_adqWriteBuffer)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiBufferSizeValues,
    unsigned int* puiValues,
    unsigned int uiRepeatBuffer,
    unsigned int& uiNumberOfValuesWritten);

typedef unsigned int (__stdcall *fptr_adqWriteContinuousStart)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiBufferSizeValues,
    unsigned int* puiValues,
    unsigned int uiRepeatBuffer,
    bool bWaitForWrite,
    unsigned int uiNotifyAfterValues,
    unsigned int& uiNumberOfValuesWritten,
    ADQ_WRITE_CONTINUOUS_CALLBACK adqWriteContinuousCallback,
    void* pvWriteContinuousUserContext);

typedef unsigned int (__stdcall *fptr_adqWriteContinuous)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiBufferSizeValues,
    unsigned int* puiValues,
    unsigned int uiRepeatBuffer,
    bool bWaitForWrite,
    unsigned int& uiNumberOfValuesWritten);

typedef unsigned int (__stdcall *fptr_adqWriteContinuousStatus)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    bool bWaitForIdle,
    unsigned int& uiNumberOfValuesAcceptable,
    bool& bRunning);

typedef unsigned int (__stdcall *fptr_adqStop)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex);

typedef unsigned int (__stdcall *fptr_adqGetIRQInfo)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    SADQIRQInfo& sIRQInfo);

typedef unsigned int (__stdcall *fptr_adqEnableIRQ)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    SADQIRQEnable& sIRQEnable,
    ADQ_IRQ_CALLBACK adqIRQCallback,
    unsigned int uiSADQIRQStatusID,
    void* pvIRQUserContext);

typedef unsigned int (__stdcall *fptr_adqWaitIRQ)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    SADQIRQStatus& sIRQStatus,
    unsigned int uiTimeoutMilliseconds);

typedef unsigned int (__stdcall *fptr_adqGetIRQCount)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int& uiIRQCount);

typedef unsigned int (__stdcall *fptr_adqDisableIRQ)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex);

typedef unsigned int (__stdcall *fptr_adqDIOGetDirection)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int& uiDirection);

typedef unsigned int (__stdcall *fptr_adqDIOSetDirection)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiDirection);

typedef unsigned int (__stdcall *fptr_adqDIOGetFilterPicoseconds)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned __int64& ui64FilterPicoseconds);

typedef unsigned int (__stdcall *fptr_adqDIOSetFilterPicoseconds)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned __int64 ui64FilterPicoseconds);

typedef unsigned int (__stdcall *fptr_adqCNTRConfig)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiClockSource,
    unsigned int uiCntrMode,
    unsigned int uiStartValue);

typedef unsigned int (__stdcall *fptr_adqCNTRConfigEx)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiCNTRConfigSize, 
    unsigned char* pucCNTRConfig);

typedef unsigned int (__stdcall *fptr_adqCNTREnable)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

typedef unsigned int (__stdcall *fptr_adqCNTRDisable)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

typedef unsigned int (__stdcall *fptr_adqAISetChannelAndRange)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiChannelIndex,
    unsigned int uiRangeIndex);

typedef unsigned int (__stdcall *fptr_adqAIGetChannelHardwareRange)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiChannelIndex, 
    unsigned int& uiRangeIndex);
    
typedef unsigned int (__stdcall *fptr_adqAISetChannelList)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    SADQAIChannel* parrsChannels,
    unsigned int uiNumberOfChannels);

typedef unsigned int (__stdcall *fptr_adqAISetChannelListAndFrequency)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    SADQAIChannel* parrsChannels, 
    unsigned int uiNumberOfChannels, 
    double dNominalFrequency, 
    double& dActualFrequency);

typedef unsigned int (__stdcall *fptr_adqAIReadSingle)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiFrameLength,
    unsigned int* puiFrame);

typedef unsigned int (__stdcall *fptr_adqAIReadBuffer)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiFrameLength,
    unsigned int uiNumberOfFramesRequired,
    unsigned int* puiFrames,
    unsigned int& uiNumberOfFramesReturned);

typedef unsigned int (__stdcall *fptr_adqAIReadContinuousStart)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiNotifyAfterFrames,
    unsigned __int64 ui64StopAfterFrames,
    ADQ_AI_READ_CONTINUOUS_CALLBACK adqAIReadContinuousCallback,
    void* pvAIReadContinuousUserContext);

typedef unsigned int (__stdcall *fptr_adqAIReadContinuous)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiFrameLength,
    unsigned int uiNumberOfFramesRequired,
    unsigned int* puiFrames,
    bool bWaitForData,
    unsigned int& uiNumberOfFramesReturned);

typedef unsigned int (__stdcall *fptr_adqAIReadContinuousStatus)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    bool bWaitForIdle,
    unsigned int& uiNumberOfFramesAvailable,
    bool& bRunning);

typedef unsigned int (__stdcall *fptr_adqAnalogGetSelectedCalibration)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int& uiCalibrationID);

typedef unsigned int (__stdcall *fptr_adqAnalogSelectCalibration)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiCalibrationID);

typedef unsigned int (__stdcall *fptr_adqAISetCalibrationZero)(
	unsigned int uiBoardIndex, 
	unsigned int uiModuleIndex, 
	unsigned int uiChannelIndex, 
	unsigned int uiRangeIndex);

typedef unsigned int (__stdcall *fptr_adqAnalogGetNumberOfUserCalibrations)(
	unsigned int uiBoardIndex, 
	unsigned int uiModuleIndex, 
	unsigned int& uiNumberOfUserCalibrations);

typedef unsigned int (__stdcall *fptr_adqAnalogSetTemporaryCalibration)(
	unsigned int uiBoardIndex, 
	unsigned int uiModuleIndex, 
	unsigned int uiChannelIndex, 
	unsigned int uiRangeIndex, 
	double dNominalPhysicalValue1, 
	double dActualPhysicalValue1, 
	double dNominalPhysicalValue2, 
	double dActualPhysicalValue2);

typedef unsigned int (__stdcall *fptr_adqAnalogGetCalibrationFactorAndConstant) (
	unsigned int uiBoardIndex, 
	unsigned int uiModuleIndex, 
	unsigned int uiChannelIndex, 
	unsigned int uiRangeIndex, 
	unsigned int uiCalibrationID, 
	double& dCalibrationFactor, 
	double& dCalibrationConstant);

typedef unsigned int (__stdcall *fptr_adqI2CSetSlaveAddress)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiSlaveAddress);

typedef unsigned int(__stdcall *fptr_adqI2CRead)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiNumberOfBytesRequired,
    unsigned char* pucData,
    unsigned int& uiNumberOfBytesReturned);

typedef unsigned int(__stdcall *fptr_adqI2CWrite)(
    unsigned int uiBoardIndex,
    unsigned int uiModuleIndex,
    unsigned int uiNumberOfBytesToWrite,
    unsigned char* pucData,
    unsigned int& uiNumberOfBytesWritten);

typedef unsigned int (__stdcall *fptr_adqI2CWriteThenRead)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiNumberOfBytesToWrite, 
    unsigned char* pucWriteData, 
    unsigned int& uiNumberOfBytesWritten, 
    unsigned int uiNumberOfBytesRequired, 
    unsigned char* pucReadData, 
    unsigned int& uiNumberOfBytesReturned);

typedef unsigned int (__stdcall *fptr_adqPWMGetMaximumFrequency)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    double& dMaximumFrequency);

typedef unsigned int (__stdcall *fptr_adqPWMConfig)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    double dNominalFrequency, 
    double dNominalOnTime, 
    bool bInvert,
    bool bHardwareTrigger,
    double& dActualFrequency, 
    double& dActualOnTime);

typedef unsigned int (__stdcall *fptr_adqPWMEnable)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiTimeoutMilliseconds);

typedef unsigned int (__stdcall *fptr_adqPWMDisable)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

typedef unsigned int (__stdcall *fptr_adqFINRead)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    double& dFrequency, 
    double& dOnTime, 
    unsigned int uiTimeoutMilliseconds);

typedef unsigned int (__stdcall *fptr_adqI2SOSineConfig)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    double dNominalOutputFrequency,
    unsigned int uiSignalLeftPrecisionBits, 
    bool bSignalLeftTwosComplement, 
    double dSignalLeftSignalFrequency, 
    double dSignalLeftAmplitude,
    unsigned int uiSignalRightPrecisionBits, 
    bool bSignalRightTwosComplement, 
    double dSignalRightSignalFrequency, 
    double dSignalRightAmplitude,
    double& dActualOutputFrequency);

typedef unsigned int (__stdcall *fptr_adqI2SOSineStart)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

typedef unsigned int (__stdcall *fptr_adqI2SOSineModify(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    double dSignalLeftSignalFrequency, 
    double dSignalLeftAmplitude, 
    double dSignalRightSignalFrequency, 
    double dSignalRightAmplitude);

typedef unsigned int (__stdcall *fptr_adqI2SOSineStop)(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

typedef unsigned int (__stdcall *fptr_adqClose)(void);

//-----------------------------------------------------------------------------
// Function Prototypes
//-----------------------------------------------------------------------------
unsigned int adqGetInternalError(
    unsigned int& uiComponent, 
    unsigned int& uiFile, 
    unsigned int& uiLineNumber, 
    unsigned int& uiType, 
    unsigned int& uiErrorParameter1, 
    unsigned int& uiErrorParameter2);

unsigned int adqOpen(void);

unsigned int adqOpenEx(
    wchar_t** ppwcNameOrIPAddress);

unsigned int adqGetNumberOfUsers(
    unsigned int& uiNumberOfUsers);

unsigned int adqGetSystemInfo(
    SADQSystemInfo& sSystemInfo);

unsigned int adqQuery(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiID, 
    unsigned int uiSubId, 
    unsigned int uiBufferSize, 
    unsigned char* pucBuffer, 
    unsigned int& uiInfoSize);

unsigned int adqGetNumberOfBoards(
    unsigned int& uiNumberOfBoards);

unsigned int adqGetBoardInfo(
    unsigned int uiBoardIndex, 
    SADQBoardInfo& sBoardInfo);

unsigned int adqGetBoardPCIInfo(
    unsigned int uiBoardIndex, 
    SADQBoardPCIInfo& sBoardPCIInfo);

unsigned int adqResetBoard(
    unsigned int uiBoardIndex);

unsigned int adqGetBoardTTLVoltage(
    unsigned int uiBoardIndex, 
    unsigned int& uiVoltage);

unsigned int adqSetBoardTTLVoltage(
    unsigned int uiBoardIndex, 
    unsigned int uiVoltage);

unsigned int adqGetNumberOfModules(
    unsigned int uiBoardIndex, 
    unsigned int& uiNumberOfModules);

unsigned int adqGetModuleType(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int& uiModuleType);

unsigned int adqGetModuleInfo(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    SADQGeneralModuleInfo& sModuleInfo, 
    unsigned int uiBufferSize, 
    unsigned char* pucBuffer, 
    unsigned int& uiInfoSize);

unsigned int adqResetModule(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

unsigned int adqGetMaximumSampleFrequency(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiNumberOfChannels, 
    double& dMaximumSampleFrequency);

unsigned int adqSetFrequency(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    double dNominalFrequency, 
    double& dActualFrequency);

unsigned int adqGetHWTriggerInfo(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    SADQHWTriggerInfo& sHWTriggerInfo);

unsigned int adqEnableHWTrigger(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    SADQHWTriggerEnable& sHWTriggerEnable, 
    bool bWaitForTrigger, 
    unsigned int uiTimeoutMilliseconds);

unsigned int adqDisableHWTrigger(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

unsigned int adqReadSingle(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int& uiValue);

unsigned int adqReadBuffer(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiNumberOfValuesRequired, 
    unsigned int* puiValues, 
    unsigned int& uiNumberOfValuesReturned);

unsigned int adqReadContinuousStart(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiNotifyAfterValues, 
    unsigned __int64 ui64StopAfterValues, 
    ADQ_READ_CONTINUOUS_CALLBACK adqReadContinuousCallback, 
    void* pvReadContinuousUserContext);

unsigned int adqReadContinuous(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiNumberOfValuesRequired, 
    unsigned int* puiValues, 
    bool bWaitForData, 
    unsigned int& uiNumberOfValuesReturned);

unsigned int adqReadContinuousStatus(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    bool bWaitForIdle, 
    unsigned int& uiNumberOfValuesAvailable, 
    bool& bRunning);

unsigned int adqWriteSingle(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiValue);

unsigned int adqWriteBuffer(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiBufferSizeValues, 
    unsigned int* puiValues, 
    unsigned int uiRepeatBuffer, 
    unsigned int& uiNumberOfValuesWritten);

unsigned int adqWriteContinuousStart(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiBufferSizeValues, 
    unsigned int* puiValues, 
    unsigned int uiRepeatBuffer, 
    bool bWaitForWrite, 
    unsigned int uiNotifyAfterValues, 
    unsigned int& uiNumberOfValuesWritten, 
    ADQ_WRITE_CONTINUOUS_CALLBACK adqWriteContinuousCallback, 
    void* pvWriteContinuousUserContext);

unsigned int adqWriteContinuous(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiBufferSizeValues, 
    unsigned int* puiValues, 
    unsigned int uiRepeatBuffer, 
    bool bWaitForWrite, 
    unsigned int& uiNumberOfValuesWritten);

unsigned int adqWriteContinuousStatus(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    bool bWaitForIdle, 
    unsigned int& uiNumberOfValuesAcceptable, 
    bool& bRunning);

unsigned int adqStop(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

unsigned int adqGetIRQInfo(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    SADQIRQInfo& sIRQInfo);

unsigned int adqEnableIRQ(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    SADQIRQEnable& sIRQEnable, 
    ADQ_IRQ_CALLBACK adqIRQCallback, 
    unsigned int uiSADQIRQStatusID, 
    void* pvIRQUserContext);

unsigned int adqWaitIRQ(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    SADQIRQStatus& sIRQStatus, 
    unsigned int uiTimeoutMilliseconds);

unsigned int adqGetIRQCount(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int& uiIRQCount);

unsigned int adqDisableIRQ(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

unsigned int adqDIOGetDirection(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int& uiDirection);

unsigned int adqDIOSetDirection(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiDirection);

unsigned int adqDIOGetFilterPicoseconds(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned __int64& ui64FilterPicoseconds);

unsigned int adqDIOSetFilterPicoseconds(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned __int64 ui64FilterPicoseconds);

unsigned int adqCNTRConfig(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiClockSource, 
    unsigned int uiCntrMode, 
    unsigned int uiStartValue);

unsigned int adqCNTRConfigEx(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiCNTRConfigSize, 
    unsigned char* pucCNTRConfig);

unsigned int adqCNTREnable(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

unsigned int adqCNTRDisable(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

unsigned int adqAISetChannelAndRange(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiChannelIndex, 
    unsigned int uiRangeIndex);

unsigned int adqAIGetChannelHardwareRange(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiChannelIndex, 
    unsigned int& uiRangeIndex);

// Function adqAISetChannelList is deprecated, please use adqAISetChannelListAndFrequency instead.
unsigned int adqAISetChannelList(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    SADQAIChannel* parrsChannels, 
    unsigned int uiNumberOfChannels);

unsigned int adqAISetChannelListAndFrequency(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    SADQAIChannel* parrsChannels, 
    unsigned int uiNumberOfChannels, 
    double dNominalFrequency, 
    double& dActualFrequency);

unsigned int adqAIReadSingle(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiFrameLength, 
    unsigned int* puiFrame);

unsigned int adqAIReadBuffer(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiFrameLength, 
    unsigned int uiNumberOfFramesRequired, 
    unsigned int* puiFrames, 
    unsigned int& uiNumberOfFramesReturned);

unsigned int adqAIReadContinuousStart(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiNotifyAfterFrames, 
    unsigned __int64 ui64StopAfterFrames, 
    ADQ_AI_READ_CONTINUOUS_CALLBACK adqAIReadContinuousCallback, 
    void* pvAIReadContinuousUserContext);

unsigned int adqAIReadContinuous(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiFrameLength, 
    unsigned int uiNumberOfFramesRequired, 
    unsigned int* puiFrames, 
    bool bWaitForData, 
    unsigned int& uiNumberOfFramesReturned);

unsigned int adqAIReadContinuousStatus(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    bool bWaitForIdle, 
    unsigned int& uiNumberOfFramesAvailable, 
    bool& bRunning);

unsigned int adqAnalogGetSelectedCalibration(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int& uiCalibrationID);

unsigned int adqAnalogSelectCalibration(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiCalibrationID);

unsigned int adqAISetCalibrationZero(
	unsigned int uiBoardIndex, 
	unsigned int uiModuleIndex, 
	unsigned int uiChannelIndex, 
	unsigned int uiRangeIndex);
	
unsigned int adqAnalogGetNumberOfUserCalibrations(
	unsigned int uiBoardIndex, 
	unsigned int uiModuleIndex, 
	unsigned int& uiNumberOfUserCalibrations);

unsigned int adqAnalogSetTemporaryCalibration(
	unsigned int uiBoardIndex, 
	unsigned int uiModuleIndex, 
	unsigned int uiChannelIndex, 
	unsigned int uiRangeIndex, 
	double dNominalPhysicalValue1, 
	double dActualPhysicalValue1, 
	double dNominalPhysicalValue2, 
	double dActualPhysicalValue2);

unsigned int adqI2CSetSlaveAddress(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiSlaveAddress);

unsigned int adqI2CRead(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiNumberOfBytesRequired, 
    unsigned char* pucData, 
    unsigned int& uiNumberOfBytesReturned);

unsigned int adqI2CWrite(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiNumberOfBytesToWrite, 
    unsigned char* pucData, 
    unsigned int& uiNumberOfBytesWritten);

unsigned int adqI2CWriteThenRead(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiNumberOfBytesToWrite, 
    unsigned char* pucWriteData, 
    unsigned int& uiNumberOfBytesWritten, 
    unsigned int uiNumberOfBytesRequired, 
    unsigned char* pucReadData, 
    unsigned int& uiNumberOfBytesReturned);

unsigned int adqPWMGetMaximumFrequency(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    double& dMaximumFrequency);

unsigned int adqPWMConfig(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    double dNominalFrequency, 
    double dNominalOnTime, 
    bool bInvert,
    bool bHardwareTrigger,
    double& dActualFrequency, 
    double& dActualOnTime);

unsigned int adqPWMEnable(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    unsigned int uiTimeoutMilliseconds);

unsigned int adqPWMDisable(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

unsigned int adqFINRead(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    double& dFrequency, 
    double& dOnTime, 
    unsigned int uiTimeoutMilliseconds);

unsigned int adqI2SOSineConfig(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    double dNominalOutputFrequency,
    unsigned int uiSignalLeftPrecisionBits, 
    bool bSignalLeftTwosComplement, 
    double dSignalLeftSignalFrequency, 
    double dSignalLeftAmplitude,
    unsigned int uiSignalRightPrecisionBits, 
    bool bSignalRightTwosComplement, 
    double dSignalRightSignalFrequency, 
    double dSignalRightAmplitude,
    double& dActualOutputFrequency);

unsigned int adqI2SOSineStart(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

unsigned int adqI2SOSineModify(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex, 
    double dSignalLeftSignalFrequency, 
    double dSignalLeftAmplitude, 
    double dSignalRightSignalFrequency, 
    double dSignalRightAmplitude);

unsigned int adqI2SOSineStop(
    unsigned int uiBoardIndex, 
    unsigned int uiModuleIndex);

unsigned int adqClose(void);


#endif
